% --------------------------------------------------
%
% Main file for the computation and visualization of
% the results shown in Figures 1a and 3 in the paper.
%
% Illustrates some properties of the synthetic
% likelihood of the Ricker example. Only the
% log growth rate r is varied.
%
% --------------------------------------------------

% if not already set, set path
%startup();

allTasks={'compute','visualize'};
taskNb = 2;

EXPORT = 1;

figFolder = '../../figures/Ricker1D';
matFolder = '../../matfiles/Ricker1D';
loadFolder = [matFolder '/paper_results']; % for results in the paper

% For plotting
fillColor = 0.8*[1 1 1];
quants = [0.1 0.5 0.9]; 

task = allTasks{taskNb};

if EXPORT 
    mkdir(figFolder)
end

if strcmp(task,'compute')
    mkdir(matFolder)
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

switch task 
    
  case 'compute'
    
    %% Data
    rng('default')
    
    seed = rng;
    thetaObs = 3.8;
    Yobs = sampleDataDistribution(thetaObs,seed,1);
    Yobs = Yobs{1};
    
    save([matFolder '/obs_Y.txt'],'-ascii','-double','Yobs')
    % Yobs = load([matFolder '/obs_Y.txt']);
    
    % figure
    stem(Yobs)
    xlabel('time')
    ylabel('count')
    if EXPORT
        myexportfig([figFolder '/obs_data'])
    end
    
    %% Objective for different S
    allS = [50 500 5000 50000];
    
    % grid
    theta_min = 3.2;
    theta_max = 4.2;
    nGrid = 100;
    theta1 = linspace(theta_min,theta_max,nGrid)';
    
    rng('shuffle');
    mySeed = rng;
    val = cell(length(allS),1);
    
    parfor k=1:length(allS);
        k
        S = allS(k);
        val{k} = objective(theta1,mySeed,S,Yobs);
    end
    
    save([matFolder '/objectives_S.mat'],'theta1','val','allS')
    
    
    %% Distribution of the objective for small S
    
    S = 50;
    nTrials = 50000;
    val = cell(nTrials,1);
    nGrid = 200;
    theta1 = linspace(3.2,5,nGrid)';
    
    parfor k=1:nTrials
        k
        val{k} = objective(theta1,[],S,Yobs);
    end
    
    save([matFolder '/objective_distribution.mat'],'theta1','val','S','nTrials')
    
  
  case 'visualize'
  
    
    %% Figures with loaded results
    %
    % Gives Figures 1a and 3 in the paper
    %
    
    % observed data
    loaded_data = load([loadFolder '/obs_Y.txt']);
    stem(loaded_data)
    xlabel('time')
    ylabel('count')
    
    if EXPORT
        myexportfig([figFolder '/obs_data'])
    end
    
    % distribution of the objective
    loaded_obj = load([loadFolder '/objectives_S.mat']);
    loaded_distr = load([loadFolder '/objective_distribution.mat']);
    
    val = [loaded_distr.val{:}];
    Jbar = mean(val,2);
    J_quants = quantile(val,quants,2);
    val_largeS = loaded_obj.val{end}-min(loaded_obj.val{end})+min(J_quants(:,2));
    theta1_long = loaded_distr.theta1;
    theta1 = loaded_obj.theta1;
    [dummy index1] = min(abs(theta1_long-theta1(1)));
    [dummy index2] = min(abs(theta1_long-theta1(end)));
    
    allS = loaded_obj.allS;
    
    figure
    ha=quants_plot(gcf,theta1_long(index1:index2)',J_quants(index1:index2,:)','r',fillColor,Jbar(index1:index2));
    h(1) = plot(theta1,loaded_obj.val{1},'b');
    h(2) = plot(theta1,loaded_obj.val{2},'g');
    h(3) = plot(theta1,loaded_obj.val{3},'m');
    h(4) = plot(theta1,loaded_obj.val{4},'k');
    
    set(gca,'xlim',[theta1(1)-0.005 theta1(end)+0.005])
    set(gca,'ylim',[6 65])
    
    myLegend = char(['N = ' num2str(allS(1))]);
    for k=2:length(allS);
        myLegend = char(myLegend,char(['N = ' num2str(allS(k))]));
    end
    myLegend = char(['0.1, 0.9 quantiles, N = 50'],['Mean, N = 50'],myLegend);
    legend([ha h],myLegend)
    xlabel('log r')
    ylabel('neg log synthetic likelihood')
    
    if EXPORT
        myexportfig([figFolder '/objective_variability_with_mean'])
    end
    
    % 
    clear h;
    figure
    h(1)=plot(theta1_long(index1:index2),Jbar(index1:index2)-min(Jbar(index1:index2)),'r');
    hold on
    
    k = 2; 
    offset_adjusted = loaded_obj.val{k}-min(loaded_obj.val{k});
    h(2) = plot(theta1,offset_adjusted,'g.-');
    
    k = 3; 
    offset_adjusted = loaded_obj.val{k}-min(loaded_obj.val{k});
    h(3) = plot(theta1,offset_adjusted,'m+-');
    
    k = 4; 
    offset_adjusted = loaded_obj.val{k}-min(loaded_obj.val{k});
    h(4) = plot(theta1,offset_adjusted,'ks-');
    
    set(gca,'xlim',[3.65 3.95])
    set(gca,'ylim',[-0.05 1.5])
    xlabel('log r')
    ylabel('neg log synthetic likelihood, offset adjusted')
    legend(h,['Mean, N = ' num2str(allS(1))],...
           ['N = ' num2str(allS(2))],...
           ['N = ' num2str(allS(3))],...
           ['N = ' num2str(allS(4))])
    
    if EXPORT
        myexportfig([figFolder '/expected_J'])
    end
    
end