% --------------------------------------------------
%
% Main file for computation of visualization of
% the results shown in Figures 7, 14, 15 in the paper. 
%
% The topic is likelihood-free inference of the Ricker
% model using Bayesian optimization. Only one model
% parameter is inferred (log-growth rate).
%
% --------------------------------------------------

% if not already set, set path
%startup();

clear all

figFolder = '../../figures/Ricker1D';
PAPER=0;
if PAPER
    matFolder = '../../matfiles/Ricker1D/paper_results';
else
    matFolder = '../../matfiles/Ricker1D';
end
                                           
allTasks={'simulations','analysis','plot'};
taskNb = 1;
% simulations: performs Bayesian optimization, options are set in setOptions.m
% analysis   : analysis of the simulation results
% plot       : selected plots as shown in Figure 7 in the paper

task = allTasks{taskNb}

if strcmp(task,'analysis')
    
    if PAPER
        % For Figure 14
        %resFile = [matFolder '/point_N50_slack0_log_run03_round0.mat']; % deterministic acq rule
                                                                         
        % For Figure 15
        resFile = [matFolder '/point_N50_slack05_log_run03_round0.mat']; % stochastic acq rule
    else
        % Example
        resFile = [matFolder '/point_N50_slack05_log_run01_round0.mat'];
    end
    
end

if strcmp(task,'plot')
    % For Figure 7
    run = '02';
end

EXPORT = 0;
EXPORT_MOVIE = 0;

if EXPORT | EXPORT_MOVIE
    mkdir(figFolder)
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


switch task
    
  case 'simulations'
        
    %% Data
    rng('default')
    Yobs = load([matFolder '/obs_Y.txt']);
    thetaObs = 3.8;
    
    %seed = rng;
    %thetaObs = 3.8;
    %Yobs = sampleDataDistribution(thetaObs,seed,1);
    %Yobs = Yobs{1};
    
    %save([matFolder '/obs_Y.txt'],'-ascii','-double','Yobs')
    
    % init
    options = setOptions;
    N = options.OBJ.N;
    obj = @(theta)(objective(theta,[],N,Yobs));
    
    evidence = init(options,obj);
    
    %figure;quickPlot(evidence,options,0);
    x = evidence.X(1:evidence.nX,:);
    f = evidence.f(1:evidence.nX);
    
    % if options are adjusted, check effect on initialization
    %options = setOptions;
    %evidence = init(options,x,f);
    %figure;quickPlot(evidence,options,0);

    % run
    [thetaHat evidence options] = fastInference(obj,options,evidence);
      
  case 'analysis'
    
    load(resFile,'options');
    slashIndex = strfind(options.IO.saveFile,'/');
    if ~isempty(slashIndex)
        slashIndex = slashIndex(end);
    else
        slashIndex = 0;
    end
    fileID = options.IO.saveFile(slashIndex+1:end);
    mypath = [matFolder '/' fileID];
    
    loaded_distr = load([matFolder '/objective_distribution.mat']);
    val = [loaded_distr.val{:}];
    Jbar = mean(val,2);
    theta1 = loaded_distr.theta1; 
  
    clear myFrame;
    figure
    LOGSCALE = 0;
    
    for k=0:options.ACQ.maxRounds
        
        if k>0
            load([mypath '_round' num2str(k) '.mat'],'evidence','options','boxes','acqMinimizer')
        elseif k==0
            load([mypath '_round' num2str(k) '.mat'],'evidence','options');
        end
        
        if  exist('boxes') && k>0
            for kk=1:size(boxes,3)  
                
                %figure
                hJ=plot(theta1,Jbar,'r');
                hold on
                h=quickPlot(evidence,options,LOGSCALE,1);
                xlabel('log r')
                ylabel('neg log synthetic likelihood')
                
                mylim = get(gca,'xlim');
                u = linspace(mylim(1),mylim(2),100)';
                acquisitionCost = acqCost(pastEvi,pastOptions,u);
                ha = plot(u,acquisitionCost,'g');
                
                plot(boxes(1,1,kk)*[1 1],get(gca,'ylim'),'g--')
                plot(boxes(1,2,kk)*[1 1],get(gca,'ylim'),'g--')
                plot(acqMinimizer(kk)*[1 1],get(gca,'ylim'),'g')
                
                y = evidence.f(evidence.nX);
                if options.GP.log ==1
                    y = exp(y);
                end
                
                if ~EXPORT
                    plot(evidence.X(evidence.nX),y,'mx','markersize',10,'linewidth',3)
                end
                
             end
            clear acqMinimizer; clear boxes;
        else
            %figure
            hJ=plot(theta1,Jbar,'r');
            hold on
            h=quickPlot(evidence,options,LOGSCALE,1);
            xlabel('log r')
            ylabel('neg log synthetic likelihood')
        end
            
        pastEvi = evidence;
        pastOptions = options;
        set(gca,'xlim',[3.2 4.2])
        set(gca,'ylim',[5 50])
         
        title(['Data points: ' num2str(evidence.nX)])
        % pause(0.5)    
        set(gcf,'color',[1 1 1])
        myFrame(k+1) = getframe(gcf);
        
        if EXPORT
            fileName = [figFolder '/BO_' fileID '_round' num2str(k)];
            myexportfig(fileName)
        end
           
        clf
    end
    
       if EXPORT_MOVIE
           fileName = [figFolder '/video_' fileID];
           writerObj = VideoWriter(fileName,'Motion JPEG AVI');
           writerObj.FrameRate = 2;
           writerObj.Quality = 100;
           open(writerObj);
           for index = 1:length(myFrame)
               writeVideo(writerObj,myFrame(index));
           end
           close(writerObj);
       end
       
  case 'plot'

    %% Selected plots
    
        
    res1_0 = load([matFolder '/point_N50_slack0_nolog_run' run '_round0.mat'],'evidence','options');
    res1_1 = load([matFolder '/point_N50_slack0_nolog_run' run '_round1.mat'],'acqMinimizer','boxes');

    res2_0 = load([matFolder '/point_N50_slack05_nolog_run' run '_round0.mat'],'evidence','options');
    res2_1 = load([matFolder '/point_N50_slack05_nolog_run' run '_round1.mat'],'acqMinimizer','boxes');

    res3_0 = load([matFolder '/point_N50_slack0_log_run' run '_round0.mat'],'evidence','options');
    res3_1 = load([matFolder '/point_N50_slack0_log_run' run '_round1.mat'],'acqMinimizer','boxes');

    res4_0 = load([matFolder '/point_N50_slack05_log_run' run '_round0.mat'],'evidence','options');
    res4_1 = load([matFolder '/point_N50_slack05_log_run' run '_round1.mat'],'acqMinimizer','boxes');


    loaded_distr = load([matFolder '/objective_distribution.mat']);
    val = [loaded_distr.val{:}];
    Jbar = mean(val,2); clear val
    theta = loaded_distr.theta1;
  
    % check whether starting points are the same
    figure
    nIni = 9;
    plot(res1_0.evidence.X(1:nIni),res1_0.evidence.f(1:nIni),'.')
    hold on
    plot(res2_0.evidence.X(1:nIni),res2_0.evidence.f(1:nIni),'ro')
    plot(res3_0.evidence.X(1:nIni),exp(res3_0.evidence.f(1:nIni)),'gs')
    plot(res4_0.evidence.X(1:nIni),exp(res4_0.evidence.f(1:nIni)),'m+')
    
    % after initialization

    % GP
    figure
    hJ=plot(theta,Jbar,'r');
    hold on
    h=quickPlot(res1_0.evidence,res1_0.options,0,1,[],0,0);
    xlabel('log r')
    ylabel('neg log synthetic likelihood')
    set(gca,'ylim',[-10 230])
    set(gca,'xlim',[3 5])
    plot(res1_1.acqMinimizer(1)*[1 1],get(gca,'ylim'),'g-')
    plot(res2_1.boxes(1)*[1 1],get(gca,'ylim'),'g--')
    plot(res2_1.boxes(2)*[1 1],get(gca,'ylim'),'g--')
    legend([hJ h],'mean, N = 50','data','post mean','0.1, 0.9 quantiles','location','northwest')
    title('')
    if EXPORT
        myexportfig([figFolder '/B0_N50_run' run '_nolog_0'])
    end
    
    % log GP
    figure
    hJ=plot(theta,Jbar,'r');
    hold on
    h=quickPlot(res3_0.evidence,res3_0.options,0,1,[],0,0);
    xlabel('log r')
    ylabel('neg log synthetic likelihood')
    set(gca,'ylim',[-10 230])
    set(gca,'xlim',[3 5])
    plot(res3_1.acqMinimizer(1)*[1 1],get(gca,'ylim'),'g-')
    plot(res4_1.boxes(1)*[1 1],get(gca,'ylim'),'g--')
    plot(res4_1.boxes(2)*[1 1],get(gca,'ylim'),'g--')
    legend([hJ h],'mean, N = 50','data','post mean','0.1, 0.9 quantiles','location','northwest')
    title('')
    if EXPORT
        myexportfig([figFolder '/B0_N50_run' run '_with_log_0'])
    end
    
    % middle
    
    res1_m = load([matFolder '/point_N50_slack0_nolog_run' run '_round15.mat'],'evidence','options');
    res2_m = load([matFolder '/point_N50_slack05_nolog_run' run '_round15.mat'],'evidence','options');
    res3_m = load([matFolder '/point_N50_slack0_log_run' run '_round15.mat'],'evidence','options');
    res4_m = load([matFolder '/point_N50_slack05_log_run' run '_round15.mat'],'evidence','options');
    
    % GP
    figure
    hJ=plot(theta,Jbar,'r');
    hold on
    h=quickPlot(res1_m.evidence,res1_m.options,0,1,[],0,0);
    xlabel('log r')
    ylabel('neg log synthetic likelihood')
    set(gca,'xlim',[3.2 4.2])
    set(gca,'ylim',[5 40])
    legend([hJ h],'mean, N = 50','data','post mean','0.1, 0.9 quantiles','location','northwest')
    legend off
    if EXPORT
        myexportfig([figFolder '/B0_N50_slack0_run' run '_nolog_middle'])
    end
    
    figure
    hJ=plot(theta,Jbar,'r');
    hold on
    h=quickPlot(res2_m.evidence,res2_m.options,0,1,[],0,0);
    xlabel('log r')
    ylabel('neg log synthetic likelihood')
    set(gca,'xlim',[3.2 4.2])
    set(gca,'ylim',[5 40])
    legend([hJ h],'mean, N = 50','data','post mean','0.1, 0.9 quantiles','location','northwest')
    legend off
    if EXPORT
        myexportfig([figFolder '/B0_N50_slack05_run' run '_nolog_middle'])
    end
    
    % log GP
    figure
    hJ=plot(theta,Jbar,'r');
    hold on
    h=quickPlot(res3_m.evidence,res3_m.options,0,1,[],0,0);
    xlabel('log r')
    ylabel('neg log synthetic likelihood')
    set(gca,'xlim',[3.2 4.2])
    set(gca,'ylim',[5 40])
    legend([hJ h],'mean, N = 50','data','post mean','0.1, 0.9 quantiles','location','northwest')
    legend off
    if EXPORT
        myexportfig([figFolder '/B0_N50_slack0_run' run '_with_log_middle'])
    end
    
    figure
    hJ=plot(theta,Jbar,'r');
    hold on
    h=quickPlot(res4_m.evidence,res4_m.options,0,1,[],0,0);
    xlabel('log r')
    ylabel('neg log synthetic likelihood')
    set(gca,'xlim',[3.2 4.2])
    set(gca,'ylim',[5 40])
    legend([hJ h],'mean, N = 50','data','post mean','0.1, 0.9 quantiles','location','northwest')
    legend off
    if EXPORT
        myexportfig([figFolder '/B0_N50_slack05_run' run '_with_log_middle'])
    end

end