% --------------------------------------------------
%
% SABC(objfun,prior,T,options)
%
% objfun: function handler pointing to a function taking theta as input and returning the distance
% prior.rnd : function handler for random number generation. Take sample size as input and returns the samples (dim x number of samples). Omit if
% options.loadFile is specified.
% prior.pdf: function handler to the pdf of the prior (takes dim x 1 vector as input and returns a scalar)
% T : number of iterations in the sequential ABC
% options.quant_start : starting quantile to determine epsilon of the first iteration (default: 0.75). Not needed if options.loadFile is specified
% options.epsi_quant : quantile for choosing next epsilon (default: 0.1)
% options.fileName : where to save the results after each iteration (default:  ['tmp_result_SABC ' date]);
% options.loadFile : possible file where an initial values of 'theta','weights','myDist','epsi' are stored. If not specified, the first iteration
% is rejection sampling.
% options.dim : dimensionality of the parameter
% options.sampleSize : number of samples to generate
% options.epsiMin : the smallest allowed threshold value (default: 0)
%
% Implements algorithm 4 "Likelihood-free population Monte Carlo sampler" of the article
% Marin, J.-M.; Pudlo, P.; Robert, C. & Ryder, R. 
% Approximate Bayesian computational methods Statistics and Computing, 
% Springer Netherlands, 2011, 1-14 
%
% --------------------------------------------------

function SABC(objfun,prior,T,options)

%% Default options

if ~isfield(options,'quant_start')
    quant_start = 0.75;
else
    quant_start = options.quant_start;
end

if ~isfield(options,'epsi_quant')
    epsi_quant = 0.1;
else
    epsi_quant = options.epsi_quant;
end


if ~isfield(options,'epsiMin')
    epsiMin = 0;
else
    epsiMin = options.epsiMin;
end

if ~isfield(options,'epsiSched') 
    epsiSched = zeros(T,1);
else
    epsiSched = options.epsiSched;
end


if ~isfield(options,'fileName')
    fileName = ['tmp_result_SABC ' date];
else
    fileName = options.fileName;
end

N = options.sampleSize;
p = options.dim;

allEpsi=cell(T,1);
allTheta=cell(T,1);
allWeights=cell(T,1);
allDist = cell(T,1);
allCost = cell(T,1);

%%Initial Sampling Step t=1

if ~isfield(options,'loadFile')

    t=1;
    theta = zeros(p,N);
    myDist = zeros(1,N);
    
    % Determine good starting epsilon
    theta_prop = zeros(p,N);
    myDist_prop = zeros(1,N);
    parfor ind=1:N
        fprintf(['\n' num2str(ind/N) '\n'])
        
        theta_prop(:,ind) = prior.rnd(1);
        myDist_prop(1,ind) = objfun(theta_prop(:,ind));
    end
        
    epsi = max(epsiMin,quantile(myDist_prop,quant_start));
    index = find(myDist_prop<epsi);
    nAccepted = length(index);
    theta(:,1:nAccepted) = theta_prop(:,index);
    myDist(1,1:nAccepted) = myDist_prop(1,index);
    nSimulations = zeros(1,N);
    nSimulations(1,1:nAccepted) = 1;
    
    %figure
    %hold on
    %plot(theta_prop,myDist_prop,'.')
    
    
    parfor ind=nAccepted+1:N
        
        fprintf(['\n' num2str(ind/N) '\n'])
        
        myDist_prop=epsi+1;
        theta_prop = [];
        tmpCounter = 0;
        while (myDist_prop>epsi)   

            theta_prop = prior.rnd(1);
            myDist_prop = objfun(theta_prop);
            tmpCounter = tmpCounter+1;
            %plot(theta_prop,myDist_prop,'.')
            %drawnow
        end
        nSimulations(1,ind) = tmpCounter;
        theta(:,ind)=theta_prop;
        myDist(1,ind) = myDist_prop;
    end
    
    weights = 1/N*ones(1,N);
        
    % Save
    allTheta{t} = theta;
    allWeights{t} = weights;
    allDist{t} = myDist;
    allEpsi{t} = epsi;
    allCost{t} = nSimulations;
    
    save(fileName,'allTheta','allWeights','allDist','allEpsi','allCost','t','options');

    tStart = 2;
else
    loaded = load(options.loadFile,'allTheta','allWeights','allDist', ...
                  'allEpsi','allCost','t');
    for kt = 1:loaded.t
        allTheta{kt} = loaded.allTheta{kt};
        allWeights{kt} = loaded.allWeights{kt};
        allDist{kt} = loaded.allDist{kt};
        allEpsi{kt} = loaded.allEpsi{kt};
        allCost{kt} = loaded.allCost{kt};
    end
        tStart = loaded.t+1; clear loaded;
end
    
%% Remaining iterations

for t=tStart:T
   
    % take the more conservative epsilon
    epsi = max(epsiSched(t),quantile(allDist{t-1},epsi_quant));
    epsi = max(epsiMin,epsi); 
    
    [Sigma dummy] = weighted_cov(allTheta{t-1},allWeights{t-1});
    Sigma = 2*Sigma;
    
    theta = zeros(p,N);
    myDist = zeros(1,N);
    weights = zeros(1,N);
    nSimulations = zeros(1,N);
    
    %  figure
    %hold on
    parfor ind=1:N
        
        fprintf(['\n' num2str(ind/N) '\n'])
        
        myDist_prop=epsi+1;
        theta_prop = [];
        tmpCounter = 0;
        while (myDist_prop>epsi)   
            index = sum(mnrnd(1,allWeights{t-1}).*[1:N]);
            theta_prop = mvnrnd(allTheta{t-1}(:,index)',Sigma,1)';
            
            if prior.pdf(theta_prop)>0
                myDist_prop = objfun(theta_prop);
                tmpCounter = tmpCounter + 1;
            end
            % plot(theta_prop,myDist_prop,'.')
            % drawnow
        end
        theta(:,ind) = theta_prop;
        myDist(1,ind) = myDist_prop;
        nSimulations(1,ind) = tmpCounter;
        
        weights(ind) = prior.pdf(theta_prop)/( allWeights{t-1}*mvnpdf(theta_prop',allTheta{t-1}',Sigma) );
        
    end

    allWeights{t}=weights/sum(weights);
    allTheta{t}=theta;
    allDist{t} = myDist;
    allEpsi{t} = epsi;
    allCost{t} = nSimulations;
    save(fileName,'allTheta','allWeights','allDist','allEpsi','allCost','t','options');
end

