% --------------------------------------------------
%
% Main file for computation and visualization of
% the results shown in Figures 11 and 12
% in the paper. (for the PMC-ABC computations performed 
% on the cluster see the files cluster_*)
%
% The topic is likelihood-free inference of the full
% day care center model using Bayesian optimization. 
%
% --------------------------------------------------


clear all

% run startup(); manually to set the path
%startup();

figFolder = '../../figures/bacterial_infections';

PAPER = 0;
if PAPER
    matFolder = '../../matfiles/bacterial_infections/paper_results';
else
    matFolder = '../../matfiles/bacterial_infections';
end

allTasks = {'simulations',...              % runs BOLFI
            'plot_evolution',...           % visualizes the progress of BOLFI as more data is acquired
            'comp_posterior',...           % computates the model-based posterior
            'make_figures_in_paper',...    % makes Figures 11 and 12 in the paper
           };

% Choose task
taskNb = 1;
task = allTasks{taskNb}

% Task-dependent settings
if strcmp(task,'simulations')
    iterStart = 1;
    iterStop = 1;
    N = 1;
    reqDataPoints_in = 90;
end

if strcmp(task,'plot_evolution')
    
    %myFile = [matFolder '/point_N1_slack05_log0_reqDataPoint90_run03_1']; % submission
    myFile = [matFolder '/point_N1_slack05_log0_reqDataPoint90_test_1'];   % paper
    
end

if strcmp(task,'comp_posterior')
    
    if PAPER
        %myFile = [matFolder '/point_N1_slack05_log0_reqDataPoint90_run03_1']; % submission
        myFile = [matFolder '/point_N1_slack05_log0_reqDataPoint90_test_1'];   % paper

        % rounds for which to compute the posterior
        myRound = [1:12]; 
    else
        % example
        myFile = [matFolder '/point_N1_slack05_log0_reqDataPoint90_test_1'];
        myRound = [1:4];
    end
        
end

if strcmp(task,'make_figures_in_paper')
    
    if PAPER
        %myFile = 'point_N1_slack05_log0_reqDataPoint90_run03_1'; % submission
        myFile = 'point_N1_slack05_log0_reqDataPoint90_test_1';   % paper
        startRound = 1;
        endRound = 12;
        maxRound = 12;
        EXPORT =1;
        refDir = '../../matfiles/bacterial_infections/paper_results/reference_cluster/';
    else
        myFile = 'point_N1_slack05_log0_reqDataPoint90_test_1';  
        startRound = 1;
        endRound = 4;
        maxRound = 4;
        EXPORT =0;
        refDir = '../../matfiles/bacterial_infections/paper_results/reference_cluster/';
    end
    
    if EXPORT
        mkdir(figFolder)
    end
    
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

switch task
    
  case 'simulations'

    % fixed parameters
    load([matFolder '/binaryRealData.mat'],'sample_sizes','global_strain_distribution')
    
    % sampled individuals in each day care center (DCC)
    fixedParams.DCC_sample_sizes = sample_sizes; clear sample_sizes;
    
    % Overall distribution of the different strains in the data
    fixedParams.global_strain_distribution = global_strain_distribution/sum(global_strain_distribution); clear global_strain_distribution;
    
    % average number of subjects in the community/group
    fixedParams.avg_DCC_size = 53; 
    fixedParams.simulation_time = 10;

    rng('shuffle')
    
    load([matFolder '/binaryRealData.mat'],'data_epi')
    sumStatsObs =  expertSummaryStatistics(data_epi,fixedParams);

    for kIter = iterStart:iterStop
        
        
        saveFileTag = ['test_' num2str(kIter)];
        % init
        options = setOptions(saveFileTag,N,reqDataPoints_in);
        obj = @(theta)(objective(theta,N,sumStatsObs,fixedParams));
    
        evidence = init(options,obj);
    
        %figure;quickPlot(evidence,options,1);
        %figure;quickPlot(evidence,options,2);
        %figure;quickPlot(evidence,options,3);   
        x = evidence.X(1:evidence.nX,:);
        f = evidence.f(1:evidence.nX);
        
        % if options are adjusted, check effect on initialization
        %options = setOptions(saveFileTag,N,reqDataPoints_in);
        %evidence = init(options,x,f);
        %figure;quickPlot(evidence,options,1);
        %figure;quickPlot(evidence,options,2);
        %figure;quickPlot(evidence,options,3);   
        
        
        % run
        [thetaHat evidence options] = fastInference(obj,options,evidence);

    end
      
  case 'plot_evolution'
      
    load([myFile '_round0.mat'],'options');    
    slashIndex = strfind(options.IO.saveFile,'/');
    if ~isempty(slashIndex)
        slashIndex = slashIndex(end);
    else
        slashIndex = 0;
    end
    fileID = options.IO.saveFile(slashIndex+1:end);
    mypath = [matFolder '/' fileID];
    
    LOGSCALE = 0;
    
    for k=0:options.ACQ.maxRounds
          
          load([mypath '_round' num2str(k) '.mat'],'evidence','options')
          figNb=figure('position', [800, 200, 900, 900]);
          clf;
          for dim1=1:options.GP.dim
              subplot(3,3,3*(dim1-1)+dim1)
              quickPlot(evidence,options,LOGSCALE,dim1);                      
          end
          for dim1=1:options.GP.dim
              for dim2=dim1+1:options.GP.dim
                  subplot(3,3,3*(dim1-1)+dim2)
                  quickPlot2D(evidence,options,[dim1 dim2],LOGSCALE)
              end
          end
          pause(0.5)
    end
        
    
  case 'comp_posterior'
    
    for kRound = 1:length(myRound)
            
            resFileBase = [myFile '_round' num2str(myRound(kRound))];

            resFile = [resFileBase '.mat'];
            load(resFile,'evidence','options');
            
            likOptions.bandwidth = [];
            likOptions.method = 'GP-uniform';
            samplingOptions.method = 'SMC';
            samplingOptions.nSamples = 25000;
            samplingOptions.priorPdf = @(u)(priorPdf(u,options.ACQ.optim.bounds));
            samplingOptions.nGenerations = 3;
            samplingOptions.fileName = [resFileBase '_posterior.mat'];            
            
            sample_posterior(evidence,options,likOptions,samplingOptions);
    end


    
  case 'make_figures_in_paper'
    
    myFilePath = [matFolder '/' myFile];
    
    %% Reference 
    
    % Reference: Table 1 in Numminen2013
    ABC_post_mean = [3.589;0.593;0.097];
    ABC_LC = [2.8157;0.4017;0.0605];
    ABC_UC = [4.5785;0.8359;0.1422];
    
    % Reference: Own simulations
    ref0 = load([refDir 'SABC_gen0.mat']);
    ref1 = load([refDir 'SABC_gen1.mat']);
    ref2 = load([refDir 'SABC_gen2.mat']);
    ref3 = load([refDir 'SABC_gen3.mat']);
    ref4 = load([refDir 'SABC_gen4.mat']);
 
    qL = 0.025;
    qU = 0.975;
    [ref_mu ref_LC ref_UC totalCost costs] = getRefNumbers(ref0,ref1,ref2,ref3,ref4,qL,qU);
    
    %% estimated regression function
    % Figure 11
    
    myLim = [2 40];
    for round=1:maxRound
        if EXPORT
            fileName = [figFolder '/' myFile 'regression_round_' num2str(round)];
        else
            fileName = '';
        end
        res = load([myFilePath '_round' num2str(round) '.mat']);
        plotLearnedModel(res.evidence,res.options,fileName,myLim);
        fprintf(['Round ' num2str(round) ...
                 ' nb of points: ' num2str(res.evidence.nX) ...
                 ' thetaHat = ' num2str(res.evidence.thetaHat(round+1,:)) '\n'])
    end
    
    
    %% marginals, posterior mean and credibility intervals
    % Figure 12
    
    % grid for evaluation of marginals
    nGrid = 100;
    res = load([myFilePath '_round1.mat']);
    bounds = res.options.ACQ.optim.bounds;
    
    nRounds = endRound-startRound+1;
    ccOffset = 4;
    cc=flipud(gray(nRounds+ccOffset));
    myMean = zeros(3,nRounds);
    myLCI = zeros(3,nRounds);
    myUCI = zeros(3,nRounds);
    myThetaJ = zeros(3,nRounds);

    % marginals
    fig1 = figure;
    plot(bounds(1,:),1/diff(bounds(1,:))*[1 1],'color',cc(ccOffset,:))
    fig2 = figure;
    plot(bounds(2,:),1/diff(bounds(2,:))*[1 1],'color',cc(ccOffset,:))
    fig3 = figure;
    plot(bounds(3,:),1/diff(bounds(3,:))*[1 1],'color',cc(ccOffset,:))
    counter = 1; 
    cc = cc(ccOffset+1:end,:);
    for round=startRound:endRound
        res = load([myFilePath '_round' num2str(round) '.mat']);
        myThetaJ(:,counter) = getThetaJ(res.evidence);
        
        loadFile = [myFilePath '_round' num2str(round) '_posterior.mat'];
        [all_post_mean CI all_hist lower_edges] = getMarginals(loadFile,bounds,[0.025 0.975],nGrid);
        myMean(:,counter) = all_post_mean;
        myLCI(:,counter) = CI(:,1);
        myUCI(:,counter) = CI(:,2);
        plotMarginals(all_post_mean, CI, all_hist, lower_edges,[fig1 fig2 fig3],cc(counter,:));
        counter = counter+1;
    end
    nSims = res.options.ACQ.nInitPoints+cumsum(res.options.ACQ.maxAcqPerRound);
    nSims = nSims(startRound:endRound);   
    str = 'prior';
    for k=1:length(nSims)
        str = char(str,[num2str(nSims(k)) ' sims']);
    end
    
    figure(fig1)  
    legend(str)
    set(gca,'xlim',[bounds(1,1) bounds(1,2)])
    xlabel('\beta')
    ylabel('estimated posterior pdf')
    set(gca,'ylim',[0 0.45])
    
    if EXPORT
        fileName = [figFolder '/' myFile '_model_posterior_pdf_beta'];
        myexportfig(fileName)
    end
    
    figure(fig2)
    legend(str)
    set(gca,'xlim',[bounds(2,1) bounds(2,2)])
    xlabel('\Lambda')
    ylabel('estimated posterior pdf') 
    
    if EXPORT
        fileName = [figFolder '/' myFile '_model_posterior_pdf_lambda'];
        myexportfig(fileName)
    end
   
    figure(fig3) 
    legend(str)
    set(gca,'xlim',[bounds(3,1) bounds(3,2)])
    xlabel('\theta')
    ylabel('estimated posterior pdf') 
    set(gca,'ylim',[0 11])
    
    if EXPORT
        fileName = [figFolder '/' myFile '_model_posterior_pdf_theta'];
        myexportfig(fileName)
    end
           
    % estimated posterior mean and CI as a function of number of simulations
    log_nSims = log10(nSims);
    log_cost = log10(totalCost);
    minix = min(log_nSims);
    maxix = max(log_cost);
    
    shading1 = [232 180 180]/255;
    %shading2 = [209 209 209]/255;
    shading2 = [200 200 200]/255; 
    
    for k=1:3
        
        figNb = figure;
        hold on
       
        mini = 0.95*min(min(myLCI(k,1:end)),ABC_LC(k));
        maxi = 1.05*max(max(myUCI(k,1:end)),ABC_UC(k));
             
        % matlab fill command spills over the limits of the axes. Correct for that.
        y = [myLCI(k,:); myMean(k,:); myUCI(k,:)];
        index = find(y(:)<mini);
        y(index) = mini;
        index = find(y(:)>maxi);
        y(index) = maxi;
        h2 = plotShaded(figNb,log_nSims,y,shading1,'ro-');
        
        h1=plot(log_nSims,myThetaJ(k,:),'gs-');
        
        yPMC = [ref_LC(k,:);ref_mu(k,:);ref_UC(k,:)];
        index = find(yPMC(:)<mini);
        yPMC(index) = mini;
        index = find(yPMC(:)>maxi);
        yPMC(index) = maxi;
        h3 = plotShaded(figNb,log_cost,yPMC,shading2,'kd-');

        yRef = [ABC_LC(k)*[1 1 1]; ABC_post_mean(k)*[1 1 1]; ; ABC_UC(k)*[1 1 1]];                
        plot([minix (maxix+minix)/2 maxix],yRef(1,:),'b^--')
        plot([minix (maxix+minix)/2 maxix],yRef(3,:),'b^--');
        h4=plot([minix (maxix+minix)/2 maxix],yRef(2,:),'b^-');
   
        set(gca,'xlim',[minix*0.995 maxix*1.005])
        set(gca,'ylim',[mini*0.995 maxi*1.005])
  
        legend([h1 h2(2) h3(2) h4],'minimizer of regression function',...
               'model-based posterior',...
               'PMC-ABC posterior',...
               'Numminen et al (final result)',...
               'location','northwest')  
                
        if k==1
            ylabel('\beta')
        elseif k==2
            ylabel('\Lambda')
        elseif k==3
            ylabel('\theta')
        end
        xlabel('Log 10 number of simulated data sets') 
        
        if EXPORT
            fileName = [figFolder '/' myFile '_model_post_mean_CI_parameter' num2str(k)];
            myexportfig(fileName)
        end
    end
    
end
  
  
 