% --------------------------------------------------
%
% Main file for the computation and visualization of
% the results shown in Figure 1b and Figure 5 in the paper.
%
% Deals with inference of the day care center example,
% one parameter at a time, using a sample-based approach.
% 
% The code is best run section by section, copying 
% it to the command line.
%
% --------------------------------------------------

clear all

startup();

figFolder = '../../figures/bacterial_infections1D';

PAPER = 1;
if PAPER
    matFolder = '../../matfiles/bacterial_infections1D/paper_results';
else
    matFolder = '../../matfiles/bacterial_infections1D';
end

LOAD_SUMSTATS = 0;
LOAD_DISTANCES = 0;

SAVE = 0;
EXPORT = 1;

if EXPORT
    mkdir(figFolder)
end

%% Set parameters of the model

% sample sizes and global strain distribution from observed data
genOptions.dirName = matFolder;
load([genOptions.dirName '/binaryRealData.mat'],'sample_sizes','global_strain_distribution')

% sampled individuals in each day care center (DCC)
fixedParams.DCC_sample_sizes = sample_sizes; 
clear sample_sizes;

% Overall distribution of the different strains in the data
fixedParams.global_strain_distribution = global_strain_distribution/sum(global_strain_distribution); 
clear global_strain_distribution;

% average number of subjects in the community/group
fixedParams.avg_DCC_size = 53; 

fixedParams.simulation_time = 10;
genOptions.fixedParams = fixedParams; clear fixedParams;

% [beta lambda theta]
% beta: intra-community hazard in (0 11)
% lambda: community hazard in (0 2)
% theta: scales proba of coinfection (0 1)
genOptions.theta = [3.6 0.6 0.1]; % estimated mean (rounded, from table 1 in Numminen2013)

%% Show samples from the model
identityFun = @(theta)(theta);    
rng('default')
seed = rng;

[data data_epi index_out remIndex_out] = sampleData(genOptions.theta,[],genOptions.fixedParams,seed,[],[],[],identityFun);


for nb=1:length(data_epi)

    myDCC = data_epi{nb};
    
    figure
    imagesc(-myDCC)
    colormap gray
    axis equal
    axis tight
    xlabel('Individual')
    ylabel('Strain')
    
    if EXPORT
        myexportfig([figFolder '/DCC_example_data_' num2str(nb)])
    end
end


%% get data
% NOTE: It would be better save the summary statistics only. Takes less space
%sumStats_be{m,k} = expertSummaryStatistics(data_be{m,k},genOptions.fixedParams);
%sumStats_la{m,k} = expertSummaryStatistics(data_la{m,k},genOptions.fixedParams);
%sumStats_th{m,k} = expertSummaryStatistics(data_th{m,k},genOptions.fixedParams);

rng('shuffle');

M = 500;
nSpace = 50;
beta = linspace(1.5,5.5,nSpace);
lambda = linspace(0.1,1.2,nSpace);
theta = linspace(0.005,0.2,nSpace);

data_be = cell(M,nSpace);
data_la = cell(M,nSpace);
data_th = cell(M,nSpace);

identityFun = @(u)(u);

for m=1:M
    m
    seed = rng;
  

    for k=1:length(beta)
        mytheta = [beta(k) genOptions.theta(2:3)];
        [data data_epi index_out remIndex_out] = sampleData(mytheta,[],genOptions.fixedParams,seed,[],[],[],identityFun);
        data_be{m,k} = data_epi; clear data_epi;
    end
    
    for k=1:length(lambda)
        mytheta = [genOptions.theta(1) lambda(k) genOptions.theta(3)];   
        [data data_epi index_out remIndex_out] = sampleData(mytheta,[],genOptions.fixedParams,seed,[],[],[],identityFun);
        data_la{m,k} = data_epi; clear data_epi;
    end
     
    for k=1:length(theta)
        mytheta = [genOptions.theta(1:2) theta(k)];    
        [data data_epi index_out remIndex_out] = sampleData(mytheta,[],genOptions.fixedParams,seed,[],[],[],identityFun);
        data_th{m,k} = data_epi; clear data_epi;
    end

end

if SAVE
    save([matFolder '/illustrate_nonparametric_approach_' num2str(date) '.mat'],'genOptions','nSpace','beta','lambda','theta','data_be','data_la','data_th')
end

%% Data for computation of efficiency
% Data is used for Example 8, gathered five sets containing M=1000 data sets each
M = 1000;

identityFun = @(u)(u);

sumStats_be = cell(1,M);
allbeta = zeros(M,1);
for m=1:M
    m
    mybeta = 11*rand;
    allbeta(m) = mybeta;
    mytheta = [mybeta genOptions.theta(2:3)];
    [data data_epi index_out remIndex_out] = sampleData(mytheta,[],genOptions.fixedParams,[],[],[],[],identityFun);
    sumStats_be{m} = expertSummaryStatistics(data_epi,genOptions.fixedParams);
end

if SAVE
    save illustrate_nonparametric_approach2_efficiency_e.mat sumStats_be allbeta
end


%% Compute distances

load([matFolder '/illustrate_nonparametric_approach2.mat'],'genOptions','nSpace','beta','lambda','theta')

% extract summary statistics from observed data
load([matFolder '/binaryRealData.mat'],'data_epi')
sumStatsObs =  expertSummaryStatistics(data_epi,genOptions.fixedParams);

% compute or load summary statistics for simulated data
if LOAD_SUMSTATS
    load([matFolder '/illustrate_nonparametric_approach2_sumStats.mat']) ; % results of data -> sumStats computation
end
if LOAD_DISTANCES
    load([matFolder '/illustrate_nonparametric_approach2_distances.mat']); % results of sumStats -> D_be,... computation
end


if ~LOAD_SUMSTATS
    
    load([matFolder '/illustrate_nonparametric_approach2.mat'],'data_be','data_la','data_th') 

    M = 300;   
    sumStats_be = cell(M,nSpace);
    sumStats_la = cell(M,nSpace);
    sumStats_th = cell(M,nSpace);
    
    for m=1:M
        
        for k=1:nSpace
            sumStats_be{m,k} = expertSummaryStatistics(data_be{m,k},genOptions.fixedParams);
            sumStats_la{m,k} = expertSummaryStatistics(data_la{m,k},genOptions.fixedParams);
            sumStats_th{m,k} = expertSummaryStatistics(data_th{m,k},genOptions.fixedParams);
        end
        
    end

    clear data_be; clear data_la; clear data_th;
end

if ~LOAD_DISTANCES
    % Compute distances
    D_be = zeros(M,nSpace);
    D_la = zeros(M,nSpace);
    D_th = zeros(M,nSpace);
    
    for m=1:M
        for k=1:nSpace
            D_be(m,k) = myExpertDistance(sumStats_be{m,k},sumStatsObs);
            D_la(m,k) = myExpertDistance(sumStats_la{m,k},sumStatsObs);
            D_th(m,k) = myExpertDistance(sumStats_th{m,k},sumStatsObs);
        end
    end
    
end
    

%% Visualize distribution of distances (discrepancies) and likelihood approximation

fillColor = 0.8*[1 1 1];
quants = [0.1 0.5 0.9];
Dq_be = quantile(D_be,quants,1);
Dm_be = mean(D_be,1);

Dq_la = quantile(D_la,quants,1);
Dm_la = mean(D_la,1);

Dq_th = quantile(D_th,quants,1);
Dm_th = mean(D_th,1);

% uniform kernel
h_be = quantile(D_be(:),0.1);
h_la = quantile(D_la(:),0.1);
h_th = quantile(D_th(:),0.1);

Lu_be = zeros(1,nSpace);
Lu_la = zeros(1,nSpace);
Lu_th = zeros(1,nSpace);

M = size(D_be,1);
for k=1:nSpace
    Lu_be(k) = sum(D_be(:,k)<h_be)/M;
    Lu_la(k) = sum(D_la(:,k)<h_la)/M;
    Lu_th(k) = sum(D_th(:,k)<h_th)/M;
end

Lbe = Lu_be/max(Lu_be)*max(Dm_be);
Lla = Lu_la/max(Lu_la)*max(Dm_la);
Lth = Lu_th/max(Lu_th)*max(Dm_th);

if SAVE
    save([matFolder '/illustrate_nonparametric_approach2_Lu.mat'],'Lu_be','Lu_la','Lu_th');
end

figure
ha1=quants_plot(gcf,beta,Dq_be,'r',fillColor,Dm_be);
ha2 =plot(beta,Lbe,'k-o');
set(gca,'xlim',[beta(1)-0.1 beta(end)+0.1])
ha3 = plot(get(gca,'xlim'),h_be*[1 1],'g');
xlabel('\beta')

if EXPORT
    myexportfig([figFolder '/illustrate_nonpara_beta'])
end

figure
ha1=quants_plot(gcf,lambda,Dq_la,'r',fillColor,Dm_la);
ha2=plot(lambda,Lla,'k-o');
ha3 = plot([beta(1) beta(end)],h_be*[1 1],'g');
set(gca,'xlim',[lambda(1)-0.01 lambda(end)+0.01])
ha3 = plot(get(gca,'xlim'),h_la*[1 1],'g');
xlabel('\Lambda')
legend([ha1 ha2 ha3],'0.1, 0.9 quantiles','average discrepancy',...
       char('approx likelihood','(rescaled)'),'threshold h')

if EXPORT
    myexportfig([figFolder '/illustrate_nonpara_lambda'])
end

figure
ha1=quants_plot(gcf,theta,Dq_th,'r',fillColor,Dm_th);
ha2=plot(theta,Lth,'k-o');
set(gca,'xlim',[theta(1)-0.005 theta(end)+0.005])
ha3 = plot(get(gca,'xlim'),h_th*[1 1],'g');
xlabel('\theta')

if EXPORT
    myexportfig([figFolder '/illustrate_nonpara_theta'])
end

%% Efficiency

% efficiency for beta on (1.5 5.5)
weights_be = zeros(1,nSpace);
weights_la = zeros(1,nSpace);
weights_th = zeros(1,nSpace);
for k=1:nSpace
    weights_be(k) = sum(D_be(:,k)<h_be);
    weights_la(k) = sum(D_la(:,k)<h_la);
    weights_th(k) = sum(D_th(:,k)<h_th);
end

weights_be = weights_be/sum(weights_be);
weights_la = weights_la/sum(weights_la);
weights_th = weights_th/sum(weights_th);

M_be = 1/(sum(weights_be.^2))
M_la = 1/(sum(weights_la.^2))
M_th = 1/(sum(weights_th.^2))

M_be/nSpace*100

M_la/nSpace*100
M_th/nSpace*100


% efficiency for beta uniform on (0 11)
loaded_a = load([matFolder '/illustrate_nonparametric_approach2_efficiency_a.mat'],'sumStats_be','allbeta');
loaded_b = load([matFolder '/illustrate_nonparametric_approach2_efficiency_b.mat'],'sumStats_be','allbeta');
loaded_c = load([matFolder '/illustrate_nonparametric_approach2_efficiency_c.mat'],'sumStats_be','allbeta');
loaded_d = load([matFolder '/illustrate_nonparametric_approach2_efficiency_d.mat'],'sumStats_be','allbeta');
loaded_e = load([matFolder '/illustrate_nonparametric_approach2_efficiency_e.mat'],'sumStats_be','allbeta');

D_be_sampled = zeros(length(loaded_a.allbeta),5);
for k=1:length(loaded_a.allbeta)
    D_be_sampled(k,1) = myExpertDistance(loaded_a.sumStats_be{k},sumStatsObs);
    D_be_sampled(k,2) = myExpertDistance(loaded_b.sumStats_be{k},sumStatsObs);
    D_be_sampled(k,3) = myExpertDistance(loaded_c.sumStats_be{k},sumStatsObs);
    D_be_sampled(k,4) = myExpertDistance(loaded_d.sumStats_be{k},sumStatsObs);
    D_be_sampled(k,5) = myExpertDistance(loaded_e.sumStats_be{k},sumStatsObs);
end
beta_sampled = [loaded_a.allbeta loaded_b.allbeta loaded_c.allbeta loaded_d.allbeta loaded_e.allbeta];

D_be_sampled = D_be_sampled(:);
beta_sampled = beta_sampled(:);

weights_sampled = D_be_sampled<h_be;

weights_sampled = weights_sampled/sum(weights_sampled);
M_be_sampled = 1/sum(weights_sampled.^2);

M_be_sampled/length(D_be_sampled)*100

index = (beta_sampled(:)<5.25) & (beta_sampled(:)>2);

wsel = weights_sampled(index);
wsel = wsel/sum(wsel);
Msel = 1/sum(wsel.^2);
Msel / sum(index)*100

(Msel/sum(index)) / (M_be_sampled/length(D_be_sampled))

index = (beta_sampled(:)>5.5) | (beta_sampled(:)<1.5);
wsel = weights_sampled(index);
sum(wsel)/length(wsel)