% --------------------------------------------------
%
% Main file for the computation and visualization of
% the results shown in Figure 8 in the paper.
%
% Deals with inference of the day care center example,
% one parameter at a time, using Bayesian optimization.
% 
% --------------------------------------------------


% set path if not already set
% startup();

clear all

figFolder = '../../figures/bacterial_infections1D';
PAPER = 0;
if PAPER
    % for results in the paper
    matFolder = '../../matfiles/bacterial_infections1D/paper_results';
else
    matFolder = '../../matfiles/bacterial_infections1D';
end

EXPORT = 0;

allTasks = {'simulations',...     % runs BOLFI
            'plot_evolution',...  % visualizes the progress of BOLFI as more data is acquired
            'plot_final'};        % makes Figure 8 in the paper

taskNb = 1;
task = allTasks{taskNb};

if strcmp(task,'simulations')
    variDim = 1;
    iterStart = 1
    iterStop = 1
end

if strcmp(task,'plot_evolution')

    % Example
    myFile = [matFolder '/point_N1_slack05_log0_reqDataPointInf_variDim_1_run01_1_round0.mat'];
end

if EXPORT
    mkdir(figFolder)
end

rng('default')

%% Load and set fixed parameters

N = 1;
reqDataPoints_in = Inf;

load([matFolder '/binaryRealData.mat'],'sample_sizes','global_strain_distribution')

% sampled individuals in each day care center (DCC)
fixedParams.DCC_sample_sizes = sample_sizes; clear sample_sizes;

% Overall distribution of the different strains in the data
fixedParams.global_strain_distribution = global_strain_distribution/sum(global_strain_distribution); clear global_strain_distribution;

% average number of subjects in the community/group
fixedParams.avg_DCC_size = 53; 
fixedParams.simulation_time = 10;


thetaFixed = [3.6 0.6 0.1];

%% The tasks

switch task
    
  case 'simulations'
    
    rng('shuffle')
    
    load([matFolder '/binaryRealData.mat'],'data_epi')
    sumStatsObs =  expertSummaryStatistics(data_epi,fixedParams);

  
    for kIter = iterStart:iterStop
        
        saveFileTag = ['variDim_' num2str(variDim) '_run01_' num2str(kIter)];
        % init
        options = setOptions(saveFileTag,N,reqDataPoints_in,variDim);
        obj = @(theta)(objective(theta,thetaFixed,variDim,N,sumStatsObs,fixedParams));
    
        evidence = init(options,obj);
    
        %figure;quickPlot(evidence,options,0);
        %x = evidence.X(1:evidence.nX,:);
        %f = evidence.f(1:evidence.nX);
        
        % if options are adjusted, check effect on initialization
        %options = setOptions(saveFileTag,N,reqDataPoints_in,variDim);
        %evidence = init(options,x,f);
        %figure;quickPlot(evidence,options,0);
     
        % run
        [thetaHat evidence options] = fastInference(obj,options,evidence);

    end
    
    case 'plot_evolution'

      load(myFile,'options');
      slashIndex = strfind(options.IO.saveFile,'/');
      if ~isempty(slashIndex)
          slashIndex = slashIndex(end);
      else
          slashIndex = 0;
      end
      fileID = options.IO.saveFile(slashIndex+1:end);
      mypath = [matFolder '/' fileID];
        
      figNb=figure;
      for k=0:options.ACQ.maxRounds
          clf
          k
          load([mypath '_round' num2str(k) '.mat'],'evidence','options')
          
          figure(figNb)
          quickPlot(evidence,options,0,1);                      
          pause(0.5)
      end
      
  case 'plot_final'
   
    loadedDist = load([matFolder '/illustrate_nonparametric_approach2_distances.mat']);
    loadedLik = load([matFolder '/illustrate_nonparametric_approach2_Lu.mat'],'Lu_be','Lu_la','Lu_th');

    res1 = load([matFolder '/point_N1_slack05_log0_reqDataPointInf_variDim_1_run03_1_round2.mat']);
    res2 = load([matFolder '/point_N1_slack05_log0_reqDataPointInf_variDim_2_run02_1_round2.mat']);
    res3 = load([matFolder '/point_N1_slack05_log0_reqDataPointInf_variDim_3_run03_1_round2.mat']);

    fillColor = 0.8*[1 1 1];
    quants = [0.1 0.5 0.9];
    Dq_be = quantile(loadedDist.D_be,quants,1);
    Dm_be = mean(loadedDist.D_be,1);
    Dq_la = quantile(loadedDist.D_la,quants,1);
    Dm_la = mean(loadedDist.D_la,1);
    Dq_th = quantile(loadedDist.D_th,quants,1);
    Dm_th = mean(loadedDist.D_th,1);
    
    u1 = [0.1:0.1:10]';
    h_be = 1.1624; % thresholds from main_01.m
    logLik1 = logLikApprox(res1.evidence,res1.options,u1,h_be,'GP-uniform');
   
    u2 = [0.05:0.01:2]';
    h_la = 1.1836;
    logLik2 = logLikApprox(res2.evidence,res2.options,u2,h_la,'GP-uniform');
 
    u3 = [0.01:0.005:1]';
    h_th = 1.1961; 
    logLik3 = logLikApprox(res3.evidence,res3.options,u3,h_th,'GP-uniform');
 
    LOGSCALE=0;
    % beta
    figure
    h1=quants_plot(gcf,loadedDist.beta,Dq_be,'r',fillColor,Dm_be);
    h2=quickPlot(res1.evidence,res1.options,LOGSCALE);
    %h3 = plot(get(gca,'xlim'),h_be*[1 1],'g');
    xlabel('\beta')
    ylabel('discrepancy')
    set(gca,'xlim',[0 9])
    set(gca,'ylim',[0.7 6])
    %legend([h1 h2 h3],'empirical 0.1, 0.9 quantiles','empirical mean','data','post mean','model 0.1, 0.9 quantiles','threshold','location','northwest')
    legend([h1 h2],'empirical 0.1, 0.9 quantiles','empirical mean','data','post mean','model 0.1, 0.9 quantiles','location','northwest')
    title('')  
    if EXPORT
        myexportfig([figFolder '/objective_beta'])
    end
    
    figure
    h1=plot(loadedDist.beta,loadedLik.Lu_be/max(loadedLik.Lu_be),'k-o');
    hold on
    h2=plot(u1,exp(logLik1-max(logLik1)),'b-');
    xlabel('\beta')
    ylabel('approximate likelihood')
    set(gca,'xlim',[loadedDist.beta(1)-0.1 loadedDist.beta(end)+0.1])
    legend([h1 h2],'sample-based','model-based')    
    if EXPORT
        myexportfig([figFolder '/lik_beta'])
    end
    
    % lambda
    figure
    h1=quants_plot(gcf,loadedDist.lambda,Dq_la,'r',fillColor,Dm_la);
    h2=quickPlot(res2.evidence,res2.options,LOGSCALE);
    %h3 = plot(get(gca,'xlim'),h_la*[1 1],'g');
    xlabel('\Lambda')
    ylabel('discrepancy')
    set(gca,'xlim',[0.05 1.85])
    set(gca,'ylim',[0.6 8])
    %legend([h1 h2 h3],'empirical 0.1, 0.9 quantiles','empirical mean','data','post mean','model 0.1, 0.9 quantiles','threshold','location','northeast')
    legend([h1 h2],'empirical 0.1, 0.9 quantiles','empirical mean','data','post mean','model 0.1, 0.9 quantiles','location','northeast')
    if EXPORT
        myexportfig([figFolder '/objective_lambda'])
    end
    
    figure
    h1=plot(loadedDist.lambda,loadedLik.Lu_la/max(loadedLik.Lu_la),'k-o');
    hold on
    h2=plot(u2,exp(logLik2-max(logLik2)),'b-');
    xlabel('\Lambda')
    ylabel('approximate likelihood')
    set(gca,'xlim',[loadedDist.lambda(1)-0.01 loadedDist.lambda(end)+0.01])
    legend([h1 h2],'sample-based','model-based')     
    if EXPORT
        myexportfig([figFolder '/lik_lambda'])
    end
    
    % theta
    figure
    h1=quants_plot(gcf,loadedDist.theta,Dq_th,'r',fillColor,Dm_th);
    h2=quickPlot(res3.evidence,res3.options,LOGSCALE);
    %h3 = plot(get(gca,'xlim'),h_th*[1 1],'g');
    xlabel('\theta')
    ylabel('discrepancy')
    set(gca,'xlim',[0 0.9])
    set(gca,'ylim',[0 25])
    %legend([h1 h2 h3],'empirical 0.1, 0.9 quantiles','empirical mean','data','post mean','model 0.1, 0.9 quantiles','threshold','location','northwest')
    legend([h1 h2],'empirical 0.1, 0.9 quantiles','empirical mean','data','post mean','model 0.1, 0.9 quantiles','location','northwest')
    if EXPORT
        myexportfig([figFolder '/objective_theta'])
    end
    
    figure
    h1=plot(loadedDist.theta,loadedLik.Lu_th/max(loadedLik.Lu_th),'k-o');
    hold on
    h2=plot(u3,exp(logLik3-max(logLik3)),'b-');
    xlabel('\theta')
    ylabel('approximate likelihood') 
    set(gca,'xlim',[loadedDist.theta(1)-0.005 loadedDist.theta(end)+0.005])
    legend([h1 h2],'sample-based','model-based')     
    if EXPORT
        myexportfig([figFolder '/lik_theta'])
    end
    

    
end