function log_expected_improvement = log_exp_imp(f_min, mu, sigma)
%sigma(find(sigma<1e-10)) = NaN; % To avoid division by zero.
% Tom Minka's normpdf takes row vectors -- column vectors are 
% interpreted as multivariate.
expected_improvement = (f_min-mu) .* normcdf((f_min-mu)./sigma) + sigma .* normpdf(((f_min-mu)./sigma)')';

%Expected improvement often yields zero. Use more robust log expected improvement
%instead.

x = (f_min-mu)./sigma;
log_expected_improvement = zeros(length(mu),1);
for i=1:length(mu)
    if abs(f_min-mu(i)) == 0
        % Degenerate case 1: first term vanishes.
        if sigma(i) > 0
            log_expected_improvement(i) = log(sigma(i)) + normpdfln(x(i));
        else
            log_expected_improvement(i) = -inf;
        end
    elseif sigma(i) == 0 
        % Degenerate case 2: second term vanishes and first term has a special form.
        if mu(i) < f_min
            log_expected_improvement(i) = log(f_min-mu(i));
        else
            log_expected_improvement(i) = -inf;
        end
    else 
        % Normal case.
        b = log(sigma(i)) + normpdfln(x(i));
        % log(y+z) is tricky, we distinguish two cases:
        if f_min>mu(i)
            % When y>0, z>0, we define a=ln(y), b=ln(z).
            % Then y+z = exp[ max(a,b) + ln(1 + exp(-|b-a|)) ],
            % and thus log(y+z) = max(a,b) + ln(1 + exp(-|b-a|))
            a = log(f_min-mu(i)) + normcdfln(x(i));
            log_expected_improvement(i) = max(a,b) + log(1 + exp(-abs(b-a)));
        else
            % When y<0, z>0, we define a=ln(-y), b=ln(z), 
            % and it has to be true that b >= a in order to satisfy y+z>=0.
            % Then y+z = exp[ b + ln(exp(b-a) -1) ],
            % and thus log(y+z) = a + ln(exp(b-a) -1)
            a = log(mu(i)-f_min) + normcdfln(x(i));
            if a >= b 
            % a>b can only happen due to numerical inaccuracies or 
            % approximation errors
                log_expected_improvement(i) = -inf;
            else
                log_expected_improvement(i) = b + log(1-exp(a-b));
            end
        end
    end
end
log_expected_improvement(find(log_expected_improvement==-inf)) = -1e100;


function e = normcdfln(x)
% NORMCDFLN   log of normal cumulative density function.
% More accurate than log(normcdf(x)) when x is small.
% The following is a quick and dirty approximation to normcdfln:
% normcdfln(x) =approx -(log(1+exp(0.88-x))/1.5)^2

% Written by Tom Minka
% (c) Microsoft Corporation. All rights reserved.

% make e the same shape as x, and inherit any NaNs.
e = x;
t = -6.5;
i = find(x >= t);
if ~isempty(i)
  e(i) = log(normcdf(x(i)));
end
i = find(x < t);
if ~isempty(i)
  x = x(i);
  z = x.^(-2);
  if 0
    % log of asymptotic series for cdf
    % subs(x=-x,asympt(sqrt(2*Pi)*gauss_cdf(-x),x));
    c = [-1 3 -15 105 -945 10395 -135135 2027025 -34459425 654729075];
    y = 0;
    for i = length(c):-1:1
      y = z.*(y + c(i));
    end
    %y = z.*(c(1)+z.*(c(2)+z.*(c(3)+z.*(c(4)+z.*(c(5)+z.*(c(6)+z.*c(7)))))));
    y = log(1+y);
  else
    % asymptotic series for logcdf
    % subs(x=-x,asympt(log(gauss_cdf(-x)),x));
    c = [-1 5/2 -37/3 353/4 -4081/5 55205/6 -854197/7];
    y = z.*(c(1)+z.*(c(2)+z.*(c(3)+z.*(c(4)+z.*(c(5)+z.*(c(6)+z.*c(7)))))));
  end
  e(i) = y -0.5*log(2*pi) -0.5*x.^2 - log(-x);
end

function p = normpdfln(x, m, S, V)
% NORMPDFLN    log of multivariate normal density.
%   See NORMPDF for argument description.

log2pi = 1.83787706640935;
[d, n] = size(x);
if nargin == 1
  dx = x;
elseif isempty(m)
  dx = x;
else
  % m specified
  sz = size(m);
  if sz(1) ~= d
    error('rows(m) ~= rows(x)')
  end
  nm = sz(2);
  if nm == 1
    dx = x - repmat(m,1,n);
  elseif n == 1
    dx = repmat(x,1,nm) - m;
  elseif nm == n
    dx = x - m;
  else
    error('incompatible number of columns in x and m')
  end
end
if nargin < 3
  % unit variance
  p = -0.5*(d*log2pi + col_sum(dx.*dx));
  return
end
have_inv = 0;
if nargin == 3
  % standard deviation given
  if d == 1
    dx = dx./S;
    p = (-log(S) -0.5*log2pi) - 0.5*(dx.*dx);
    return;
  end
  if S(2,1) ~= 0
    error('S is not upper triangular')
  end
  if any(size(S) ~= [d d])
    error('S is not the right size')
  end
else
  if ischar(V)
    if strcmp(V,'inv')
      % inverse stddev given
      iS = S;
      have_inv = 1;
    else
      error('unknown directive')
    end
  elseif ischar(S) 
    if strcmp(S,'inv')
      % inverse variance given
      if d == 1
	iS = sqrt(V);
      else
	iS = chol(V);
      end
      have_inv = 1;
    else
      error('unknown directive')
    end
  else
    % variance given
    if d == 1
      S = sqrt(V);
    else
      S = chol(V);
    end
  end
end
if have_inv
  if d == 1
    dx = iS .* dx;
    logdetiS = log(iS);
  else
    dx = iS*dx;
    logdetiS = sum(log(diag(iS)));
  end
else
  if d == 1
    dx = dx./S;
    logdetiS = -log(S);
  else
    dx = solve_tril(S',dx);
    %dx = S'\dx;
    logdetiS = -sum(log(diag(S)));
  end
end
p = (logdetiS -0.5*d*log2pi) -0.5*col_sum(dx.*dx);

function s = col_sum(x)
% COL_SUM   Sum for each column.
% A more readable alternative to sum(x,1).
s = sum(x,1);
