% --------------------------------------------------
%
%  varargout = compute_post_mean_var(evidence,optionsGP,x,mode)
%
% x(k,:) contains row vector for k-th query
% returned gradients are row vectors as well.
%
%  mode:
%   'mean'           :  varargout{1} = postMean
%   'mean-var'       : 
%        varargout{1} = postMean;
%        varargout{2} = diag(postVar_latent); 
%        varargout{3} = diag(postVar);
%    'mean-covar'       : 
%        varargout{1} = postMean;
%        varargout{2} = postVar_latent;
%        varargout{3} = postVar;
%   'mean-var-k'     :
%        varargout{1} = postMean;
%        varargout{2} = diag(postVar_latent);
%        varargout{3} = diag(postVar);
%        varargout{4} = k_x;
%        varargout{5} = diag(k_xx);
%    'mean-grad'     :
%        varargout{1} = postMean;
%        varargout{2} = postMean_grad;
%    'mean-var-grad' :
%        varargout{1} = postMean;
%        varargout{2} = postVar_latent; 
%        varargout{3} = postVar;
%        varargout{4} = postMean_grad;
%        varargout{5} = postVar_grad;
%
% If gradient is required, only marginal variances are computed.
%
% --------------------------------------------------


function varargout = compute_post_mean_var(evidence,optionsGP,x,mode)
        
    
    if strcmp(mode,'mean')
        GRADIENTS = 0;
        VAR = 0;
    elseif strcmp(mode,'mean-var') | strcmp(mode,'mean-var-k') |  strcmp(mode,'mean-covar')
        GRADIENTS = 0;
        VAR = 1;
    elseif strcmp(mode,'mean-grad')
        GRADIENTS = 1;
        VAR = 0;
    elseif strcmp(mode,'mean-var-grad')
        GRADIENTS = 1;
        VAR = 1;
    end
            
    if ~GRADIENTS
        priorMean  = prior_mean(evidence.hyp.mean,optionsGP.mean,x);   
        k_x = feval(optionsGP.cov{:},evidence.hyp.cov,evidence.X(1:evidence.nX,:),x);
        k_xx = feval(optionsGP.cov{:},evidence.hyp.cov, x,x);
        postMean = priorMean + k_x'*evidence.alpha; 
    
        if VAR
            V  = evidence.L\k_x; 
            % postVar_latent = max(k_xx - sum(V.*V,1)',eps);
            postVar_latent = k_xx - V'*V;
            d = diag(postVar_latent);
            d = max(d,eps);
            
            postVar_latent = postVar_latent-diag(diag(postVar_latent))+diag(d);
            postVar = postVar_latent + eye(length(d))*exp(2*evidence.hyp.lik);
        end
    
    end
    
    if GRADIENTS
        
        %fprintf('compute gradient \n')
        [priorMean priorMean_grad] = prior_mean(evidence.hyp.mean,optionsGP.mean,x);   
        nPoints = size(x,1);
        dim = size(x,2);
        postMean = zeros(nPoints,1);
        postMean_grad = zeros(nPoints,dim);
        
        if VAR
            postVar_latent = zeros(nPoints,1);
            postVar = zeros(nPoints,1);
            postVar_grad = zeros(nPoints,dim);
        end
       
        for k=1:nPoints
            [k_x k_xgrad] = feval(optionsGP.cov{:},evidence.hyp.cov,evidence.X(1:evidence.nX,:),x(k,:));
            [k_xx k_xxgrad] = feval(optionsGP.cov{:},evidence.hyp.cov, x(k,:), x(k,:));
            postMean(k,1) = priorMean(k) + k_x'*evidence.alpha; 
            postMean_grad(k,:) = priorMean_grad(k,:) + (k_xgrad*evidence.alpha)';
            
            if VAR
                V  = evidence.L\k_x;
                postVar_latent(k,1) = max(k_xx - sum(V.*V,1)',eps);
                postVar(k,1) = postVar_latent(k,1) + exp(2*evidence.hyp.lik);
                postVar_grad(k,:) = k_xxgrad' - 2*V'*(evidence.L\k_xgrad');
            end                
        end
    end
  
    
    if strcmp(mode,'mean')
        varargout{1} = postMean;
    elseif strcmp(mode,'mean-var') 
        varargout{1} = postMean;
        varargout{2} = diag(postVar_latent);
        varargout{3} = diag(postVar);
    elseif strcmp(mode,'mean-covar') 
        varargout{1} = postMean;
        varargout{2} = postVar_latent;
        varargout{3} = postVar;
    elseif strcmp(mode,'mean-var-k')
        varargout{1} = postMean;
        varargout{2} = diag(postVar_latent);
        varargout{3} = diag(postVar);
        varargout{4} = k_x;
        varargout{5} = diag(k_xx);
    elseif strcmp(mode,'mean-grad')
        varargout{1} = postMean;
        varargout{2} = postMean_grad;
    elseif strcmp(mode,'mean-var-grad')
        varargout{1} = postMean;
        varargout{2} = postVar_latent;
        varargout{3} = postVar;
        varargout{4} = postMean_grad;
        varargout{5} = postVar_grad;
    end
    