
% --------------------------------------------------
%
% Main file for the computation and visualization of
% the results shown in Figures 2 and 4 in the paper.

% The topic is the inference of the mean of a Gaussian
% in a likelihood-free setting. The file makes use of
% the analytical expressions which are available for 
% this simple example.
%
% --------------------------------------------------

figFolder = '../../figures/meanGauss1D';
EXPORT = 1;

mkdir(figFolder);


%% Data
n = 10;

rng('default')

theta0 = 1;
yobs = theta0 + randn(1,n);


%% Objective
%
% The objective is here defined as (phiobs-muHat)^2 where 
% muHat is normal distributed with mean theta and variance 1/(N*n).
% N is the number of simulated data sets and 
% n is the size of each data set.
%
% The objective is related to the negative logarithm of the synthetic likelihood,
% negative synthetic log likelihood = 0.5*log(2*pi/n) + n/2*(phiobs-muHat)^2
%

objType = {'neg_log_syn_lik','discrepancy'};

% For Figure 2a, set myType = 1;
% For Figure 4a, set myType = 2;

myType = 1;

if strcmp(objType(myType),'neg_log_syn_lik') 
  
    % set number of simulated data sets to estimate muHat
    N = 2;
  
    % transformation from objective to negative synthetic log-likelihood
    scale = n/2;
    offset = 0.5*log(2*pi/n);
    
    % labels
    myYlabel = 'neg log synthetic likelihood';
    saveString = 'neg_log_syn_lik_N';
    
elseif strcmp(objType(myType),'discrepancy')
   
    N = 1;
    scale = 1;
    offset = 0;
    
    % labels
    myYlabel = 'discrepancy';
    saveString = 'discrepancy_N';
    
end
    
% define grid where to evaluate the objective
theta = [-2:0.01:4];

% summary statistics of observed data
phiobs = mean(yobs);

% mean objective
Jbar = ( (phiobs-theta).^2+1/(N*n) );

% quantiles of the objective
p = 0.9;
uJ = zeros(size(theta));
for k=1:length(theta)
    myTheta = theta(k);
    f1 = @(J0)( normcdf( (sqrt(J0)-myTheta+phiobs)*sqrt(N*n),0,1)-normcdf((-sqrt(J0)-myTheta+phiobs)*sqrt(N*n),0,1)-p );
    uJ(k) = fzero(f1,[0 1000]);
end

p = 0.1;
lJ = zeros(size(theta));
for k=1:length(theta)
    myTheta = theta(k);
    f1 = @(J0)( normcdf(( sqrt(J0)-myTheta+phiobs)*sqrt(N*n),0,1)-normcdf((-sqrt(J0)-myTheta+phiobs)*sqrt(N*n),0,1)-p );
    lJ(k) = fzero(f1,[0 1000]);
end

% sample the objective at different theta
M = 30;
myTheta = linspace(theta(1),theta(end),M);
Jr = zeros(M,1);
for k=1:M
    g = 1/sqrt(N*n)*randn;
    Jr(k) = (phiobs-myTheta(k)-g).^2;
end

% sample a random function 
g1 =randn*1/sqrt(N*n);
J1 = (phiobs-theta-g1).^2;

% plot
figNb = figure;
qJ = [lJ;Jbar;uJ]*scale+offset;
fillColor = 0.8*[1 1 1];
ha=quants_plot(figNb,theta,qJ,'r',fillColor);

hJ = plot(theta,J1*scale+offset,'k--');
hJr = plot(myTheta,Jr*scale+offset,'ko');

xlabel('\theta')
ylabel(myYlabel)
legend([ha hJ(1) hJr(1)],'0.1, 0.9 quantiles','mean','realization of stochastic process','realizations at \theta')

set(gca,'xlim',[theta(1)-0.1 theta(end)+0.1]);
set(gca,'ylim',[-0.5 16]*scale+offset);

if EXPORT
    myexportfig([figFolder '/' saveString num2str(N)])
end

%% Conversion from discrepancy to likelihood (for N=1, i.e. objType==discrepancy)
%
% Gives Figure 4b    
%

if strcmp(objType(myType),'discrepancy')

    threshold = 0.1;
    likHat = zeros(size(theta));
    
    for k=1:length(theta)
        val = theta(k);
        likHat(k) =  normcdf( (phiobs-val+sqrt(threshold))*sqrt(n),0,1)-...
        normcdf((phiobs-val-sqrt(threshold))*sqrt(n),0,1);
        lik(k) = exp(-sum((yobs-val).^2)/2);
    end
    lik = lik/max(lik);
    likHat = likHat/max(likHat);
    
    figNb = figure;
    ha=quants_plot(figNb,theta,qJ,'r',fillColor);
    xlabel('\theta')
    
    set(gca,'xlim',[0 4]);
    set(gca,'ylim',[-0.02 1.1]);
  
    h1=plot(get(gca,'xlim'),threshold*[1 1],'g');
    h2=plot(theta,likHat,'b-');
    h3=plot(theta,lik,'k--');
    legend([ha h1 h2 h3],'0.1, 0.9 quantiles','mean',['threshold (' num2str(threshold) ')'],...
           char('approximate likelihood','(rescaled)'),char('true likelihood','(rescaled)'))
    
    if EXPORT
        myexportfig([figFolder '/likApprox'])
    end
    
end

%% Distribution of the maximizer of the synthetic likelihood
%
% Gives Figure 2b
%

N = [1 3 10];
myPlotStyle = {'b-.','g--','r-','k'};

x = phiobs+[-1:0.01:1];

figure
hold on
for k=1:length(N)
    v = 1/(N(k)*n);
    thetaHatpdf = normpdf(x,phiobs,sqrt(v));
    plot(x,thetaHatpdf,myPlotStyle{k})
end
plot(phiobs*[1 1],get(gca,'ylim'),myPlotStyle{end})

myLegend = ['N = ' num2str(N(1))];
for k=2:length(N)
    myLegend = char(myLegend,['N = ' num2str(N(k))]);
end
myLegend = char(myLegend,['N = \infty (MLE)']);

xlabel('estimate')
ylabel('pdf')
legend(myLegend)
set(gca,'xlim',[x(1) x(end)])


if EXPORT
    myexportfig([figFolder '/max_syn_lik_estimate_distribution'])
end
