% --------------------------------------------------
%
% Main file for the computation and visualization of
% the results shown in Figures 6 and 13 in the paper.
%
% The topic is the inference of the mean of a Gaussian
% in a likelihood-free setting using Bayesian 
% optimization.
%
% --------------------------------------------------

clear all

figFolder = '../../figures/meanGauss1D';

PAPER=0;
if PAPER
    baseDir = '../../matfiles/meanGauss1D/paper_results';
else
    baseDir = '../../matfiles/meanGauss1D';
    mkdir(baseDir)
end


allTasks={'BO','show_evolution','use_of_prior_info'};
taskNb = 1;
% BO: Run Bayesian optimization (set appropriate setOptions file below)
% show_evolution: Visualize the workings of Bayesian optimization. Makes the plot for Figure 6
% use_of_prior_info: Investigates the use of prior information in Bayesian optimization. Makes Figure 13.

task = allTasks{taskNb};

EXPORT = 0;
EXPORT_MOVIE = 0;

if strcmp(task,'show_evolution')
    % File to be loaded for task==show_evolution
    resFile = [baseDir '/point_N1_slack0_yobs1_const_movie_run1_round0.mat'];
end

if EXPORT
    mkdir(figFolder)
end
    
%% Data

n = 10;
maxIter = 1;

rng('default')

theta0 = 1;
yobs = theta0 + randn(1,n);

% summary statistics of observed data
phiobs = mean(yobs);

%% Tasks

switch task
    
  case 'BO'   
  
    %% Inference

    rng('shuffle') 
   
    for k=1:maxIter
       
        % constant mean
        %N=1;
        %N=2;
        %N=10; 
        %saveFileTag = ['yobs1_2Init_meanConst_run' num2str(k)];
        %options = setOptions_const(saveFileTag,N);
          
        % use of prior in acquisition
        %N=10; % assumes synthetic likelihood approach
        %saveFileTag = ['yobs1_2Init_meanConst_prior_run' num2str(k)];
        %options = setOptions_const_prior(saveFileTag,N);
       
        % movie
        N=1;
        saveFileTag = ['yobs1_const_movie_run' num2str(k)];
        options = setOptions_movie(saveFileTag,N);
       
        % define objective
        obj = @(theta)(objective(theta,N,phiobs,n));
        
        % init from scratch
        mySeed = rng;
        evidence = init(options,obj);
        
        %quickPlot(evidence,options,0);
        x = evidence.X(1:evidence.nX,:);
        f = evidence.f(1:evidence.nX);
        
        % run
        save([options.IO.saveFile '_setup.mat'],'phiobs','mySeed','yobs','theta0','x','f','mySeed')
        [thetaHat evidence options] = fastInference(obj,options,evidence);
    
    end

  case 'show_evolution'
    % Figure 6
    
    load(resFile,'options');
    clear myFrame;
    
    % reference
    theta = linspace(options.ACQ.optim.bounds(1),options.ACQ.optim.bounds(2),100);
    N = options.OBJ.N;
    Jbar = ( (phiobs-theta).^2+1/(N*n) );
    p = 0.9;
    uJ = zeros(size(theta));
    for k=1:length(theta)
        myTheta = theta(k);
        f1 = @(J0)( normcdf(( sqrt(J0)-myTheta+phiobs)*sqrt(N*n),0,1)-normcdf((-sqrt(J0)-myTheta+phiobs)*sqrt(N*n),0,1)-p );
        uJ(k) = fzero(f1,[0 1000]);
    end
    p = 0.1;
    lJ = zeros(size(theta));
    for k=1:length(theta)
        myTheta = theta(k);
        f1 = @(J0)( normcdf(( sqrt(J0)-myTheta+phiobs)*sqrt(N*n),0,1)-normcdf((-sqrt(J0)-myTheta+phiobs)*sqrt(N*n),0,1)-p );
        lJ(k) = fzero(f1,[0 1000]);
    end
  
    figNb = figure;
    qJ = [lJ;Jbar;uJ];
    fillColor = 0.8*[1 1 1];
    offset = min(Jbar)-3;
    scale = max(Jbar)-min(Jbar);
         
    slashIndex = strfind(options.IO.saveFile,'/');
    if ~isempty(slashIndex)
        slashIndex = slashIndex(end);
    else
        slashIndex = 0;
    end
    fileID = options.IO.saveFile(slashIndex+1:end);
    mypath = [baseDir '/' fileID];
  
    
    for k=0:options.ACQ.maxRounds
        
        load([mypath '_round' num2str(k) '.mat'],'evidence','options');
                
        if k>0
            lastAcqMinimizer = acqMinimizer;
            lastX = evidence.X(evidence.nX);
            lastF = evidence.f(evidence.nX);
            if options.GP.log ==1
                lastF = exp(lastF);
            end
        end
        
        figure(figNb)
        ha=quants_plot(figNb,theta,qJ,'r',fillColor,Jbar);
        hold on
        h=quickPlot(evidence,options,0);
        
        mylim = get(gca,'xlim');
        u = linspace(mylim(1),mylim(2),100)';
        acquisitionCost = acqCost(evidence,options,u);
        acquisitionCost = acquisitionCost-min(acquisitionCost);
        acquisitionCost = acquisitionCost/(max(acquisitionCost))*scale+offset;
        ha = plot(u,acquisitionCost,'m');
        [mini index]=min(acquisitionCost);
        acqMinimizer = u(index);
        plot(acqMinimizer*[1 1],get(gca,'ylim'),'m--')
         
        % show location of last acquisition
        %if k>0
        %    plot(lastAcqMinimizer*[1 1],get(gca,'ylim'),'m-.')
        %    plot(lastX,lastF,'mx','markersize',10,'linewidth',3)
        %end
       
        xlabel('\theta')
        ylabel('discrepancy')
        pastEvi = evidence;
        pastOptions = options;
        set(gca,'xlim',[-5.1 5.1])
        if k>0
            set(gca,'ylim',[-5 50])
        end
        title(['Data points: ' num2str(evidence.nX)])
        
        pause(0.5)    
        
        set(gcf,'color',[1 1 1])
        myFrame(k+1) = getframe(gcf);
        
        if EXPORT
            fileName = [figFolder '/BO_' fileID '_round' num2str(k)];
            myexportfig(fileName)
        end
           
        clf
        

    end
    
       if EXPORT_MOVIE
           fileName = [figFolder '/video_' fileID];
           writerObj = VideoWriter(fileName,'Motion JPEG AVI');
           writerObj.FrameRate = 1.5;
           writerObj.Quality = 100;
           open(writerObj);
           for index = 1:length(myFrame)
               writeVideo(writerObj,myFrame(index));
           end
           close(writerObj);
       end
       
  case 'use_of_prior_info'
    % Figure 13
    
    if PAPER
        % paper:
        res0 = load([baseDir '/point_N10_slack01_yobs1_2Init_meanConst_run6_round1.mat']);
        res1 = load([baseDir '/point_N10_slack01_yobs1_2Init_meanConst_prior_run1_round1.mat']);
    else
        % example
        res0 = load([baseDir '/point_N10_slack01_yobs1_2Init_meanConst_run1_round1.mat']);
        res1 = load([baseDir '/point_N10_slack01_yobs1_2Init_meanConst_prior_run1_round1.mat']);
    end
    
    u = [-15:0.1:10]';
    
    if EXPORT
        fileName1 = [figFolder '/use_of_prior_info_N10_noprior'];
        fileName2 = [figFolder '/use_of_prior_info_N10_prior'];
    else
        fileName1 = [];
        fileName2 = [];
    end
    
    postAnalysisPlot(res0,u,phiobs,n,fileName1);
    postAnalysisPlot(res1,u,phiobs,n,fileName2);
    
    
  otherwise
    fprintf('Invalid task \n')
    
    
end