% --------------------------------------------------
%
% sumStats = expertSummaryStatistics(DCC_sampled,fixedParams)
%
% Pieced together from code provided by E.Numminen. 
%
% sumStats: the summary statistics used by Numminen et al, 2013
% DCC_sampled: cell array of binary matrices representing the day care centers (DCCs)
% fixedParams.DCC_sample_sizes  : row vector
% fixedParams.global_strain_distribution: row vector
% fixedParams.avg_DCC_size    : scalar
% fixedParams.simulation_time : scalar
%
%
% %% Reference:
% Numminen, E.; Cheng, L.; Gyllenberg, M. and Corander, J. 
% Estimating the Transmission Dynamics of Streptococcus pneumoniae from Strain Prevalence Data, 
% Biometrics, 2013, 69, 748-757
%
% --------------------------------------------------

function sumStats = expertSummaryStatistics(DCC_sampled,fixedParams)

%   This function simulates an ensemble of distances from the observed data. 
%   The distance ensemble is such that independent simulations are performed 'N_units' times using the same model
%   parameters, determined by the input variable "parameters", and the results are compared to data consisting also of 'N_units' different units. 
%   Here the comparison is made by computing the L^1 distances of the empirical distribution functions of certain
%   summaries, computed unitwise from both simulation results and data. These are then 
%   given as an output. 



max_N_strains = length(fixedParams.global_strain_distribution);
N_units = length(fixedParams.DCC_sample_sizes);  
  
[observedstrains, observedcarriage, totalcoinfection, observedcoinfection, totalcarriage] = ...
    my_ensemble_observations(DCC_sampled,N_units,max_N_strains,fixedParams.DCC_sample_sizes);

% the following rows correspond to determining the L^1 distances of the empirical distribution functions of the distributions of the summaries calculated for each local population.  
% This section should be modified according to the summaries and
% distance_measures used. 


diversity_observations= zeros(4,N_units);               % Calculate different unitwise summaries for describing the diversity of local populations. Notice that only 1'th and 3'th summaries were currently used. 

for k= 1:N_units
    diversity_observations(1, k) = Diversity(double(observedstrains(:,k))', 1);         %Shannon index of diversity
    
    if  (sum(double(observedstrains(:,k)))>0)
        %The proportion of observed individuals that were colonized with the most prevalent strain
        diversity_observations(2, k) = max(double(observedstrains(:,k)))/(sum(double(observedstrains(:,k))));      
    end
    
    if  (sum(double(observedstrains(:,k)))==0)
        diversity_observations(2, k) = 0;
    end
    
    %How many strains were found
    diversity_observations(3, k) = length(find(observedstrains(:,k)));                                  
                                                                                                        
    %Singleton strains
    diversity_observations(4, k) = length(find(observedstrains(:,k)==1));                              
end

% summary stats are 
% (1-4) diversity_observation
% (5) observedcoinfection
% (6) observedcarriage
% (7) totalcoinfection
% (8) totalcarriage

sumStats.t1=diversity_observations(1,:);
%sumStats.t2=diversity_observations(2,:);
sumStats.t3=diversity_observations(3,:);
%sumStats.t4=diversity_observations(4,:);
sumStats.t5=observedcoinfection;
sumStats.t6=observedcarriage;
%sumStats.t7=totalcoinfection;
%sumStats.t8=totalcarriage;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function[observedstrains, observedcarriage, totalcoinfection, observedcoinfection, totalcarriage] = my_ensemble_observations(DCC_sampled,N_units,max_N_strains,sample_sizes)
       
    observedcoinfection = zeros(N_units,1);
    observedcarriage = zeros(N_units,1);
    observedstrains = zeros(max_N_strains,N_units);

    for  i=1:N_units
        observed = DCC_sampled{i};
        totalcarriage =sum(observed);
        observedcoinfection(i) = sum(totalcarriage>1)/sample_sizes(i);
        observedcarriage(i) =sum(totalcarriage>=1)/sample_sizes(i);
        observedstrains(:,i) = sum(observed,2);
    end
    
    totalcoinfection = (sum(sample_sizes.*observedcoinfection'))/sum(sample_sizes);             %over all populations, what is the prevalence of colonization
    totalcarriage = (sum(sample_sizes.*observedcarriage'))/sum(sample_sizes);                   % -"- , the prevalence of co-infection.
    
    