
function [DCC_Arr] = individual_DCC_simulation(state, simulation_time, global_strain_distribution, N_individuals, N_strains)

    % This function simulates the stochastic transmission dynamics of N_strains within a local population (DCC) of
    % avg_DCC_size individuals. Individuals are subject to infections from
    % outside the local population, i.e. from global population, where the
    % prevalence of infection and diversity distribution of co-circulating
    % strains is assumed to be constant over time, and also get infections
    % from each other. 
    
    % DCC_Arr                                   is an (N_strains x N_individuals)-array where the (i, j) element = 1 if the j'th individual in the population is infected by the i'th strain. 
    
    % 'state'                                   determines the model parameters and the initial state
    %                                           of DCC_Arr. 
    
    % 'global_strain_distribution'              determines the distribution
    %                                           of the N_strains strains in the global population, i.e. outside the
    %                                           local population simulated here. 
    
    % 'avg_DCC_size'                            determines the size of the
    %                                           local population
    
    % 'N_strains'                               determines the number of
    %                                           different strains co-circulating in the global population

intraRate = state.intraRate;
recoverRate = state.recoverRate;
alienRate = state.alienRate;
theta = state.theta;
Temp_Arr = state.TArr;                          % Array describing the exact infection profile of each member of local population (DCC), that is updated througout the simulation
infections = sum(Temp_Arr);                     % Vector describing for each individual of the local population how many strains he is colonized with (range [0, N_strains])
Infected = sum((infections>0));                 % An integer describing the number of individuals in the local population currently colonized with one or many strains.
curTime = state.time;
endTime = state.time + simulation_time;

while curTime<endTime
    
    DCC_Arr= Temp_Arr;
    tmpTime = zeros(3,1);
    tmpTime(1) = intraInfect(intraRate, Infected, N_individuals);
    tmpTime(2) = intraRecover(recoverRate, infections, N_individuals);
    tmpTime(3) = alienInfect(alienRate, N_individuals);
    [nextEventTime, minInd] = min(tmpTime(:));
    [eventType] = ind2sub(size(tmpTime),minInd);
    
    
    
    
    
    switch eventType
        case 1
            [flag NewArr NewInfected Newinfections] = update_DCC_Arr(Temp_Arr,Infected, infections, global_strain_distribution, N_individuals, N_strains, 'intrainfect');
            
        case 2
            [flag NewArr NewInfected Newinfections] = update_DCC_Arr(Temp_Arr,Infected, infections, global_strain_distribution, N_individuals, N_strains, 'decrease');
            
        case 3
            [flag NewArr NewInfected Newinfections] = update_DCC_Arr(Temp_Arr,Infected, infections, global_strain_distribution, N_individuals, N_strains, 'alieninfect');
            
        otherwise
            error('unknow event type: %d!',eventType);
    end
    
    if flag
        
        Temp_Arr = NewArr;
        curTime = curTime + nextEventTime;
        Infected = NewInfected;
        infections = Newinfections;
        
    end
end
%-------------------------------------------------------------------------%
    function [flag, NewArr, NewInfected, Newinfections] = update_DCC_Arr(OldArr, OldInfected, Oldinfections, global_strain_distribution, avg_DCC_size, N_strains, action)
        
        switch action
            
            case 'intrainfect'
                
                flag = true;
                
                if OldInfected >=2                                      %if more than two individuals are infected within a DCC, infection targets either a totally susceptible individual or already infected individual.
                    times = exprnd(1./[OldInfected*(avg_DCC_size-OldInfected)  OldInfected*(OldInfected-1)]);
                    [~,  event] = min(times);
                else
                    event=1;
                end
                
                if event==1
                    ind1 = randsample(avg_DCC_size-OldInfected,1);
                    ind2 = find(Oldinfections==0);
                    
                    
                    ind3 = randsample(OldInfected,1);
                    ind4 = find(Oldinfections>0);
                    infector = OldArr(:, ind4(ind3));
                    strain = find(infector);
                    howmanystrains= length(strain);
                    if (howmanystrains > 1)
                        ind6 = randsample(howmanystrains,1);
                        strain = strain(ind6);
                    end
                    NewArr = OldArr;
                    NewArr(strain ,ind2(ind1)) = 1;
                    NewInfected = OldInfected+1;
                    Newinfections = Oldinfections;
                    Newinfections(ind2(ind1)) = 1;
                end
                
                if event==2

                    succes= binornd(1, theta);                   % First see if co-infection is succesfull, only succesfull co-infections result in changes in the population array
                    
                    if succes ==1
                        ind1 = randsample(OldInfected, 1);
                        ind2 = randsample(OldInfected-1, 1);
                        infected = find(Oldinfections>0);
                        infector = infected(ind1);
                        infected(ind1) = [];
                        coinfected = infected(ind2);
                        infectingstrains = find(OldArr(:, infector));
                        infectingstrain =randsample(length(infectingstrains),1);
                        strain = infectingstrains(infectingstrain);
                        
                        if OldArr(strain, coinfected) ==0                   % If the target of infection doesn't have the infecting strain the population array is changed. 
                            NewArr = OldArr;
                            NewArr(strain ,coinfected) = 1;
                            NewInfected = OldInfected;
                            Newinfections = Oldinfections;
                            Newinfections(coinfected) = Oldinfections(coinfected)+1;
                            
                        else
                            NewArr = OldArr;                                % ... otherwise not.
                            NewInfected = OldInfected;
                            Newinfections = Oldinfections;
                        end
                        
                    else
                        NewArr = OldArr;
                        NewInfected = OldInfected;
                        Newinfections = Oldinfections;
                    end
                    
                end
                
            case 'decrease'
                
                if (OldInfected == 0)
                    flag = false;
                else
                    
                    flag = true;
                    strains = find(Temp_Arr>0);                         % Determine a random colonizing strain that clears the infection.
                    tmp = randsample(length(strains), 1);
                    strainstmp = strains(tmp);
                    [recoveringstrain,who] = ind2sub([N_strains, avg_DCC_size] ,strainstmp);
                    
                    clearing3 = sum(OldArr(:, who));
  
                    if clearing3== 1
                        NewArr = OldArr;
                        NewArr(:, who) = 0;
                        NewInfected = OldInfected-1;
                        Newinfections = Oldinfections;
                        Newinfections(who) = 0;
                    else
                        
                        NewArr = OldArr;
                        NewInfected = OldInfected;
                        NewArr(recoveringstrain, who) = 0;
                        Newinfections = Oldinfections;
                        Newinfections(who) = Oldinfections(who)-1;
                    end
                    
                    
                end
                
                
            case 'alieninfect'
                
                flag = true;
                times = exprnd(1./[avg_DCC_size-OldInfected OldInfected]);
                [~,  event] = min(times);
                
                ind = find(mnrnd(1,global_strain_distribution));
                
                if event ==1
                    
                    ind1 = randsample(avg_DCC_size-OldInfected,1);
                    ind2 = find(Oldinfections==0);
                    infected = ind2(ind1);
                    
                    NewArr = OldArr;
                    NewArr(ind, infected) =1;
                    NewInfected = OldInfected+1;
                    Newinfections = Oldinfections;
                    Newinfections(infected) = 1;
                    
                end
                
                if event ==2
                    
                    succes= binornd(1, theta);
                    if succes ==1
                        
                        ind1 = randsample(OldInfected,1);
                        ind2 = find(Oldinfections>0);
                        infected = ind2(ind1);
                        
                        if OldArr(ind, infected) ==0
                            
                            NewArr = OldArr ;
                            NewArr(ind, infected) = 1;
                            NewInfected = OldInfected;
                            Newinfections = Oldinfections;
                            Newinfections(infected) = Oldinfections(infected) +1;
                        else
                            NewArr =OldArr;
                            NewInfected = OldInfected;
                            Newinfections =Oldinfections;
                        end
                        
                    else
                        NewArr =OldArr;
                        NewInfected = OldInfected;
                        Newinfections =Oldinfections;
                    end
                    
                    
                end
        
            otherwise
                error('unknown action: %s', action);
        end
        
    end

%-------------------------------------------------------------------------%

    function y = intraInfect(intraRate, Infected, avg_DCC_size)
   
        avgTime = intraRate*(Infected*(avg_DCC_size-Infected)+Infected*(Infected-1))/(avg_DCC_size);
        
        if Infected==0
            y= Inf;
        end
        
        if avgTime==0
            y= Inf;
        else
            y = exprnd(1/avgTime);
        end
    end

%-------------------------------------------------------------------------%

    function y = intraRecover(recoverRate, infections, avg_DCC_size)

        avgTime = recoverRate*sum(infections);
        
        if avgTime==0
            y= Inf;
        else
            y = exprnd(1/avgTime);
        end
        
    end
%-------------------------------------------------------------------------%

    function y = alienInfect(alienRate, avg_DCC_size)
 
        y = exprnd(1./(alienRate*avg_DCC_size));
    end

%-------------------------------------------------------------------------%

end
