% --------------------------------------------------
%   
% [data data_epi index_out remIndex_out] = sampleData(parameters,sample_size, fixedParams,seed,pool,index_in, remIndex_in, invtransf)
%
%
% Simulates the spread of strains in multiple day care centers. The simulated spread is in variable "data_epi". The variable "data" is
% a data matrix which is constructed from "data_epi". The simulation of the day care centers is done by code provided by
% E. Numminen, which was used for the article:
%
% Numminen, E.; Cheng, L.; Gyllenberg, M. and Corander, J. 
% Estimating the Transmission Dynamics of Streptococcus pneumoniae from Strain Prevalence Data, 
% Biometrics, 2013, 69, 748-757
%
%
% %% General input arguments:
% seed: seed to be used for the random number generation
%       If empty, the current state of the random number generator 
%       is used.
% invtransf: transforms unbounded parameters to the right bounds. Can be identity function @(theta)(theta).
%
%
% %% Input arguments affecting output variable "data_epi":
% parameters : [beta lambda theta]
%              beta: intra-community hazard in (0 11)
%              lambda: community hazard in (0 2)
%              theta: scales proba of coinfection (0 1)
%              e.g. parameters = [3.6 0.6 0.1];
%
% fixedParams.DCC_sample_sizes  : row vector with day care center (DCC) sizes
% fixedParams.global_strain_distribution: row vector with probabilities for observing different strains
% fixedParams.avg_DCC_size    : scalar
% fixedParams.simulation_time : scalar
%
% data_epi: 1xlength(DCC_sample_sizes) cell array with binary matrices of size (length(global_strain_distribution) x DCC_sample_sizes(k) )
%
%
% %% Input arguments affecting output variable "data": 
% sample_size: number of samples to draw. If empty only "data_epi" is generated, the remaining outputs are returned []. The inputs arguments below
% can in this case be [].
% pool, index_in, remIndex_in: technical items needed by reshape_data.m. If index_in and remIndex = [], they are randomly generated and returned
% as index_out and remIndex_out. 
%
% data: The data used by the classifiers. Constructed from data_epi using reshape_data.m
%
% --------------------------------------------------

function [data data_epi index_out remIndex_out] = sampleData(parameters, sample_size, fixedParams, seed, pool, index_in, remIndex_in, invtransf)
    
    
%% Set seed
    if ~isempty(seed)
        currentSeed = rng;
        rng(seed);
    end
    
    %% Sample
    theta = invtransf(parameters);  
    data_epi = sample_epi_model(theta,fixedParams);
  
    if ~isempty(sample_size)
        mySumStats = expertSummaryStatistics(data_epi,fixedParams);
        DCCSampleSizes = ceil(fixedParams.DCC_sample_sizes/sum(fixedParams.DCC_sample_sizes)*sample_size);
        myDiff = sum(DCCSampleSizes)-sample_size;
    
        [data index_out remIndex_out] = reshape_data(data_epi,pool,DCCSampleSizes,index_in, remIndex_in,myDiff,mySumStats);
    else
        data = [];
        index_out = [];
        remIndex_out = [];
    end
    
    
    %% reset seed
    if ~isempty(seed)
        rng(currentSeed);
    end
    
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% --------------------------------------------------
%
% Interface with code from Elina Numminen
% (individual_DCC_simulation.m)
%
% --------------------------------------------------


function  DCC_sampled = sample_epi_model(theta,fixedParams)
    
    
%% Settings : fixed parameters
    
% the distribution of the strains
    global_strain_distribution = fixedParams.global_strain_distribution;

    % max number of strains in the population
    max_N_strains = length(fixedParams.global_strain_distribution);
    
    % number of DCCs
    N_units = length(fixedParams.DCC_sample_sizes);  

    % average total number of individuals in a DCC
    avg_DCC_size = fixedParams.avg_DCC_size;
        
    % number of individuals sampled in each DCC
    DCC_sample_sizes = fixedParams.DCC_sample_sizes;
    
    % time till sampling, in multiples of the clearance rate (==1) 
    simulation_time = fixedParams.simulation_time;
    
    
    %% Sample the Day Care Centers
    state = initState(theta, avg_DCC_size, max_N_strains);
    
    DCC_sampled = cell(1,N_units);

    parfor k=1:N_units
        DCC = individual_DCC_simulation(state, simulation_time, ...
                                        global_strain_distribution, avg_DCC_size, max_N_strains);
        sampled = randsample(avg_DCC_size, DCC_sample_sizes(k));
        DCC_sampled{k}= DCC(:,sampled);
    end
   