% --------------------------------------------------
%
%  thetaHat = fastInference(obj,options)
%  thetaHat = fastInference(obj,options,evidence)
%  [thetaHat evidence] = fastInference(obj,options)
%  [thetaHat evidence] = fastInference(obj,options,evidence)
%  [thetaHat evidence options] = fastInference(obj,options)
%  [thetaHat evidence options] = fastInference(obj,options,evidence)
%
%
%  obj : function handler to the objective to be
%        _minimized_
%
%  out = obj(theta), where 
%      -  theta is matrix with possibly m  _row_ vectors of parameters
%      -  out is a column vector with the m function values,  one 
%           for each row vector of parameters, or a cell with:
%           out(1) = {column vector with m function values}, 
%           out(2) = {{additional info 1}, ..., {additional info m}}
%
% The algorithm work in rounds. The results after 
% each round are saved in [options.IO.saveFile '_round' num2str(kRound)]
%
% Outputs (are all available in the saved files):
%   thetaHat : final estimate (equals evidence.thetaHat(end,:))
%   evidence : the gathered evidence using Bayesian optimization
%   options  : the options used (some bounds may be updated
%              during the algorithm, see BO/handle_exploitation.m) 
%
% --------------------------------------------------


function varargout = fastInference(obj,options,varargin)
    
   
    %% Initialization    
    fprintf('\n \n')
    fprintf('Initialize \n')
    ticStart = tic;
     
    if nargin == 2
        evidence = init(options,obj);
    elseif nargin == 3
        evidence = varargin{1};
    end
    
    if options.GP.log == 1
        obj = @(u)(logobj(obj,u));
    end
     
    [evidence options] = pointEstimate(obj,evidence,options,0);
    
    fprintf('Saving results \n')
    thetaHat = evidence.thetaHat(1,:);
    save([options.IO.saveFile '_round0'],'evidence','options','thetaHat');
    clear thetaHat;
    
    time = toc(ticStart);
    fprintf(['Initialization finished (' num2str(time) ' s)  \n'])
    
      
    %% The main working loop
   
    for kRound=1:options.ACQ.maxRounds
        
        fprintf('\n \n')
        fprintf(['Round ' num2str(kRound) '/' num2str(options.ACQ.maxRounds) '\n']);
        ticStart = tic;
        [evidence options acqMinimizer boxes] = pointEstimate(obj,evidence,options,kRound);
                
        % Saving
        fprintf(['Saving results of round ' num2str(kRound) '\n'])
        thetaHat = evidence.thetaHat(1:kRound+1,:);
        save([options.IO.saveFile '_round' num2str(kRound)],'evidence','options','thetaHat','acqMinimizer','boxes');
        clear thetaHat;
        time = toc(ticStart);
        fprintf(['Round finished:  (' num2str(time) ' s)  \n'])
        
    end

    %% Define outputs 
    
    if nargout == 1
        varargout{1} = evidence.thetaHat(end,:);
    elseif nargout == 2
        varargout{1} = evidence.thetaHat(end,:);
        varargout{2} = evidence;
    elseif nargout == 3
        varargout{1} = evidence.thetaHat(end,:);
        varargout{2} = evidence;
        varargout{3} = options;
    end
    